/**
 * @author Tim 提供js接口供html界面调用app原生功能
 */
+(function() {
	'use strict';
	var FXAPP = function(opts){
		//预留可配置能力
	};
	FXAPP.prototype = {
		// 以下参数用于回调页面使用
		Queue : [],
		onSuccess : function(id, d) {
			try {
				top.FXAPP.Queue[id].callback(d);
			} catch (e) {
			}
		},
		onError : function(id, d) {
			try {
				top.FXAPP.Queue[id].errorCallback(d);
			} catch (e) {
			}
		},
		/**
		 * 增加任务以便回调使用
		 * @param success
		 * @param fail
		 * @returns 任务序号
		 */
		addTask : function(success, fail){
			var task = {
				id : this.Queue.length,
				callback : success,
				errorCallback : fail
			};
			top.FXAPP.Queue.push(task);
			return task.id;
		},
		// 以下方法为调用原生功能的统一入口
		postMessageToApp : function(method, params, success, fail) {
			params = params == undefined ? {} : params;
			params["method"] = method;
			params["taskId"] = this.addTask(success, fail);
			if (window.fsapp && window.fsapp.postMessage) {
				window.fsapp.postMessage(JSON.stringify(params));
			} else {
				alert("请在APP上使用该功能！");
				fail && fail();
			}
		},
		//ios环境下调用
		initIosContext : function() {
			window.fsapp = {};
			fsapp.params = {};
			fsapp.method = {};
			fsapp.postMessage = function(params) {
				var paramsJson = eval("(" + params + ")");
				var key = "key" + paramsJson.taskId;
				fsapp.params[key] = params;
				fsapp.method[key] = paramsJson.method;
				var iframe = document.createElement("IFRAME");
				iframe.setAttribute("src", "foresee://callNative?key=" + key);
				document.documentElement.appendChild(iframe);
				iframe.parentNode.removeChild(iframe);
				iframe = null;
			};
		},
		//判断当前浏览器是否是app定制浏览器
		isApp : function(){
			return typeof (window.fsapp) != "undefined";
		},
		//原生提供的功能声明开始
		//post请求，发请求之前应注意浏览器cookie的同步
		//post : function(params, success, fail){
		//	this.postMessageToApp("post", params, success, fail);
		//},
		//get请求，发请求之前应注意浏览器cookie的同步
		//get : function(params, success, fail){
		//	this.postMessageToApp("get", params, success, fail);
		//},
		/**
		 * 原生接口：返回
		 */
		back : function(){
			this.postMessageToApp("back");
		},
		/**
		 * 原生接口：退出APP
		 */
		quit : function(){
			this.postMessageToApp("quit");
		},
		/**
		 * 原生接口：进入首页
		 */
		enter : function(){
			this.postMessageToApp("enter");
		},
		/**
		 * 原生接口：语音识别
		 * @param obj 对象或id
		 */
		voiceRecognition : function(params, success, fail){
			this.postMessageToApp("voiceRecognition", params, success, fail);
		},
		/**
		 * 打开小程序
		 */
		navigateToMiniProgram : function(path){
			this.postMessageToApp("navigateToMiniProgram", {path: path});
		},
		/**
		 * 原生接口：扫描二维码
		 * @param callback 扫描成功后回调
		 */
		scanQRCode : function(params, success, fail){
			this.postMessageToApp("scanQRCode", params, success, fail);
		},
		/**
		 * 原生接口：检查相机权限
		 * 
		 */
		CameraAuthority : function(params, success, fail){
			this.postMessageToApp("CameraAuthority", params, success, fail);
		},
		/**
		 * 原生接口：下载，调用手机系统系统
		 * @param url
		 */
		downloadFile : function(params, success, fail){
			this.postMessageToApp("downloadFile", params, success, fail);
		},
		/**
		 * 原生接口：下载，下载文件到指定目录，
		 * @param url
		 */
		downloadFileToLocal : function(params, success, fail){
			this.postMessageToApp("downloadFileToLocal", params, success, fail);
		},
		/**
		 * 原生接口
		 * 查看文件是否存在 
		 * @param localPath 文件路径含文件名
		 */
		fileIsExists : function(params, success, fail){
			this.postMessageToApp("fileIsExists", params, success, fail);
		},
		/**
		 * 原生接口
		 * 调用系统工具打开文件
		 * @param localPath 文件路径含文件名
		 */
		openFile : function(params, success, fail){
			this.postMessageToApp("openFile", params, success, fail);
		},
		/**
		 * 原生接口 
		 * 修改屏幕方向 (横屏<->竖屏)
		 * 
		 */
		ConvertScreenDirection:function(params, success, fail){
			this.postMessageToApp("ConvertScreenDirection", params, success, fail);
		},
		/**
		 * 原生接口 
		 * 竖屏显示
		 * 
		 */
		setScreenPortrait:function(params, success, fail){
			this.postMessageToApp("setScreenPortrait", {url:window.location.href}, success, fail);
		},
		/**
		 * 原生接口 
		 * 横屏显示
		 * 
		 */
		setScreenLandscape:function(params, success, fail){
			this.postMessageToApp("setScreenLandscape", {url:window.location.href}, success, fail);
		},
		/**
		 * 原生接口 
		 * 获取当前屏幕方向
		 * 返回结果 {Direction:'Landscape/Portrait'} Portrait:竖屏; Landscape:横屏
		 */
		getScreenDirection:function(params, success, fail){
			this.postMessageToApp("getScreenDirection",{},success, fail);
		},
		/**
		 * 获取位置信息
		 * 返回结果{}
		 */
		getLocation : function(success, fail) {
			this.postMessageToApp("getLocation", {}, success, fail);
		},
		getCacheSize : function(success, fail) {
			this.postMessageToApp("getCacheSize", {}, success, fail);
		},
		clearCache : function(success, fail) {
			this.postMessageToApp("clearCache", {}, success, fail);
		},
		getVersion : function(success, fail) {
			this.postMessageToApp("getVersion", {}, success, fail);
		},
		checkUpdate : function(success, fail) {
			this.postMessageToApp("checkUpdate", {}, success, fail);
		},
		startBaiduMap : function(params) {
			this.postMessageToApp("startBaiduMap", params);
		},
		startAMap : function(params) {
			this.postMessageToApp("startAMap", params);
		},
		/**
		 * 查看PDF
		 */
		viewPdf : function(params, success, fail) {
			this.postMessageToApp("viewPdf", params, success, fail);
		},
		/**
		 * 陕西证书登录
		 */
		SXCertLogin : function(params, success, fail) {
			this.postMessageToApp("SXCertLogin", params, success, fail);
		},
		/**
		 * 陕西证书申请
		 */
		SXCertApply : function(params, success, fail) {
			this.postMessageToApp("SXCertApply", params, success, fail);
		},
		/**
		 * 陕西证书下载
		 */
		SXCertDownload : function(params, success, fail) {
			this.postMessageToApp("SXCertDownload", params, success, fail);
		},
		/**
		 * 陕西证书安装
		 */
		SXCertInstall : function(params, success, fail) {
			this.postMessageToApp("SXCertInstall", params, success, fail);
		},
		/**
		 * 陕西证书信息
		 */
		SXCertInfo : function(params, success, fail) {
			this.postMessageToApp("SXCertInfo", params, success, fail);
		},
		/**
		 * 芝麻认证
		 */
		zhimaRenzheng : function(params, success, fail) {
			this.postMessageToApp("zhimaRenzheng", params, success, fail);
		},
		/**
		 * 实名认证（陕西）
		 */
		smrz : function(params, success, fail) {
			this.postMessageToApp("smrz", params, success, fail);
		},
		/**
		 * 陕西CA状态
		 */
		SXCAStatus : function(params, success, fail) {
			this.postMessageToApp("SXCAStatus", params, success, fail);
		},
		/**
		 * 陕西CA 数据转换
		 */
		idNoToNumber : function(params, success, fail) {
			this.postMessageToApp("idNoToNumber", params, success, fail);
		},
		/**
		 * 陕西CA 注册
		 */
		RegisterSXCA : function(params, success, fail) {
			this.postMessageToApp("RegisterSXCA", params, success, fail);
		},
		/**
		 * 陕西ca 申请证书
		 */
		ApplyCertSXCA : function(params, success, fail) {
			this.postMessageToApp("ApplyCertSXCA", params, success, fail);
		},
		/**
		 * 陕西ca 登录
		 */
		CertLoginSXCA : function(params, success, fail) {
			this.postMessageToApp("CertLoginSXCA", params, success, fail);
		},
		/**
		 * 陕西ca 登录证书校验
		 */
		appAuthSXCA : function(params, success, fail) {
			this.postMessageToApp("appAuthSXCA", params, success, fail);
		},
		/**
		 * 陕西ca 申请印模
		 */
		GetMoulageSXCA : function(params, success, fail) {
			this.postMessageToApp("GetMoulageSXCA", params, success, fail);
		},
		/**
		 * 陕西ca 生成待签名请求xml
		 */
		startSignPdfSXCA : function(params, success, fail) {
			this.postMessageToApp("startSignPdfSXCA", params, success, fail);
		},
		/**
		 * 陕西ca 签名pdf
		 */
		SignPdfSXCA : function(params, success, fail) {
			this.postMessageToApp("SignPdfSXCA", params, success, fail);
		},
		/**
		 * bsae64 pdf字符串 转 base64字符串 图片
		 */
		changeToJPG : function(params, success, fail) {
			this.postMessageToApp("changeToJPG", params, success, fail);
		},
		/**
		 * 陕西ca 获取CA用户信息
		 */
		getSXCAUserInfo : function(params, success, fail) {
			this.postMessageToApp("getSXCAUserInfo", params, success, fail);
		},
		/**
		 * 陕西ca 证书补办
		 */
		getSXCACerts : function(params, success, fail) {
			this.postMessageToApp("getSXCACerts", params, success, fail);
		},
		/**
		 * 获取APP保存的CA登录类型
		 */
		getCALoginType : function(params, success, fail) {
			this.postMessageToApp("getCALoginType", params, success, fail);
		},
		/**
		 * 河南ca 签名pdf
		 */
		signPdfHNCA : function(params, success, fail) {
			this.postMessageToApp("signPdfHNCA", params, success, fail);
		},
		setAccountToApp : function(params, success, fail) {
			this.postMessageToApp("setAccountToApp", params, success, fail);
		},
		removeAccountToApp : function(params, success, fail) {
			this.postMessageToApp("removeAccountToApp", params, success, fail);
		},
		isSupportApplePay : function(params, success, fail) {
			this.postMessageToApp("isSupportApplePay", params, success, fail);
		},
		startApplePay : function(params, success, fail) {
			this.postMessageToApp("startApplePay", params, success, fail);
		},
		startPay : function(params, success, fail) {
			this.postMessageToApp("startPay", params, success, fail);
		},
		startWeChat : function(params, success, fail) {
			this.postMessageToApp("startWeChat", params, success, fail);
		},
		sxZwAppBack: function() {
			var keys = '关闭';
			var u = navigator.userAgent;
			var deviceIsWindowsPhone = u.indexOf("Windows Phone") >= 0;
			var isiOS = /iPhone/.test(u) && !deviceIsWindowsPhone;
			var isAndroid = u.indexOf('Android') > 0 && !deviceIsWindowsPhone;
			if (isiOS) {
				window.webkit.messageHandlers.PopToApp.postMessage(keys);
			} else if (isAndroid) {
				android.PopToApp(keys);
			}
		}
		//原生提供的功能声明结束
	};
	window.FXAPP = new FXAPP();
})();