/**
 * @author Tim 移动端ui框架
 * 本ui依赖jquery
 */
+(function($) {
	'use strict';
	var FXUI = function(opts){
		// 预留可配置能力
		this.init();
		this.initEvent();
		this.setPageTitle();
	};
	String.prototype.startsWith = String.prototype.startsWith || function (str) {
	    var reg = new RegExp("^" + str);
	    return reg.test(this);
	};
	String.prototype.endsWith = String.prototype.endsWith || function (str) {
	    var reg = new RegExp(str + "$");
	    return reg.test(this);
	};
	FXUI.prototype = {
		IsWxConfig:false,
		// 以下参数用于回调页面使用
		Queue : [],
		onSuccess : function(id, d) {
			try {
				top.FXUI.Queue[id].callback(d);
			} catch (e) {
			}
		},
		onError : function(id, d) {
			try {
				top.FXUI.Queue[id].errorCallback(d);
			} catch (e) {
			}
		},
		/**
		 * 增加任务以便回调使用
		 * @param success
		 * @param fail
		 * @returns 任务序号
		 */
		addTask : function(success, fail){
			var task = {
				id : this.Queue.length,
				callback : success,
				errorCallback : fail
			};
			top.FXUI.Queue.push(task);
			return task.id;
		},
		init : function(){
			//加载移动端ui主框架light7
			try {
                var initComponent = $("script[src*='FXUI-1.0.js']").attr("initComponent");
                if(initComponent) {
                    window.console && console.log('加载初始化控件：'+initComponent);
                	this.requireComponent(initComponent);
				}else {
					this.requireComponent("light7");
				}
			}catch (e) {
				window.console && console.log(e);
				this.requireComponent("light7");
			}
			if (window.localStorage.FxAppId && $('html').attr('ng-app') != 'app') {
				this.require("/static/css/app/" + window.localStorage.FxAppId + ".css", "css");
			}
			//检测系统，自动去除苹果返回按钮
			//this.setBackBtn();
			//去除支付宝城市服务返回按钮
			this.setAliAppBackBtn();
			//检测url后面是否有debug标记，有的话就引用vconsole进行调试
			if(this.parameterForURL("debug") == "true"){
				window.localStorage.setItem("appDebug","true");
			}else if(this.parameterForURL("debug") == "false"){
				window.localStorage.removeItem("appDebug");
			}
			if(window.localStorage.getItem("appDebug") == "true"){
				this.require("/static/libs/vconsole/vconsole.min.js","js");
			}

			if (this.parameterForURL("debugMode") == "true") {
				sessionStorage.setItem("debugMode","true");
			}
			var devPort = this.parameterForURL("devPort");
			if (this.parameterForURL("devPort")) {
				sessionStorage.setItem("devPort",devPort);
			}
			// var deviceIsWx = navigator.userAgent.toLowerCase().match(/MicroMessenger/i) == "micromessenger";
			// if(deviceIsWx){
			// 	this.require("https://res.wx.qq.com/open/js/jweixin-1.3.2.js", "js");
			// 	wx.miniProgram.getEnv(function(res) {
			// 		if (res.miniprogram) {//在小程序中
			// 			$('header').addClass('mp-header');
			// 			// $(".content").css("top","0");
			// 		}
			// 	});
			// }
			var localUrl =window.location.href;
			//大部分页面未加 <meta http-equiv="Content-Security-Policy" content="upgrade-insecure-requests">
			//ios微信环境，https域名下不能加载http资源
			if(localUrl.startsWith("https") && this.isWx() ){
				//$("head").append('<meta http-equiv="Content-Security-Policy" content="upgrade-insecure-requests"/>');
				// this.require("https://res.wx.qq.com/open/js/jweixin-1.2.0.js","js");
				this.require("https://res.wx.qq.com/open/js/jweixin-1.3.2.js","js");
			}
			$(document).ready(function() {
				window.__wxjs_environment === 'miniprogram' && $('header').addClass('mp-header');
				window.localStorage.getItem("FxAppId") == '71fe12758670454099479db833d187cf' && $('header').addClass('mp-header');
			});
		},
		initEvent : function() {
			var that = this;
			this.opttime = Date.now();
			var t = 30 * 60 * 1000;
			$(document)['on']('mousedown touchstart', function(e) {
				that.isWx() && (Date.now() - that.opttime > t) && window.localStorage.verifyUrl && (top.location.href = window.localStorage.verifyUrl);
				that.opttime = Date.now();
			});
		},
		/**
		 * 加载指定ui组件
		 * @param name 组件名称，当前可选组件为"light7"、"vue"、"swiper"、"cityPicker"、"swipeout","fileupload","ajaxFileUpload"
		 */
		requireComponent : function(name){
			switch(name)
			{
				case "light7"://移动端ui主框架，必须引用
					this.require("/static/libs/light7-0.4.3/css/light7.min.css","css");
					this.require("/static/css/mylight7.css","css");
					this.require("/static/libs/light7-0.4.3/js/light7.min.js","js");
					this.require("/static/libs/light7-0.4.3/js/i18n/cn.min.js","js");
					break;
				case "light7NoCss"://移动端ui主框架，去掉light7.min.css
					this.require("/static/css/mylight7.css","css");
					this.require("/static/libs/light7-0.4.3/js/light7.min.js","js");
					this.require("/static/libs/light7-0.4.3/js/i18n/cn.min.js","js");
					break;
				case "light7WithWaterMark"://移动端ui主框架，去掉light7.min.css
                    this.require("/static/libs/light7-0.4.3/css/light7.min.css","css");
                    this.require("/static/css/mylight7.css","css");
                    this.require("/static/libs/light7-0.4.3/js/light7.js","js");
                    this.require("/static/libs/light7-0.4.3/js/i18n/cn.min.js","js");
                    this.require("/static/js/assets/watermark.js","js");
                    this.require("/static/js/assets/watermarkReadyFunc.js","js");
					break;
				case "light7Debug"://移动端ui主框架，必须引用
					this.require("/static/libs/light7-0.4.3/css/light7.css","css");
					this.require("/static/css/mylight7.css","css");
					this.require("/static/libs/light7-0.4.3/js/light7.js","js");
					this.require("/static/libs/light7-0.4.3/js/i18n/cn.min.js","js");
					break;
				case "vue"://mvvm双向数据绑定组件
					this.require("/static/libs/vue/vue.min.js","js");
					break;
				case "swiper"://light7框架的组件之一：幻灯片
					this.require("/static/libs/light7-0.4.3/css/light7-swiper.min.css","css");
					this.require("/static/libs/light7-0.4.3/js/light7-swiper.min.js","js");
					break;
				case "swiperNew"://light7框架的组件之一：幻灯片
					this.require("/static/libs/light7-0.4.3/css/light7-swiper.min.css","css");
					this.require("/static/libs/light7-0.4.3/js/swiper.js","js");
					break;
				case "cityPicker"://light7框架的组件之一：地区选择器
					this.require("/static/libs/light7-0.4.3/js/light7-city-picker.min.js","js");
					break;
				case "swipeout"://light7框架的组件之一：列表滑动删除组件
					this.require("/static/libs/light7-0.4.3/css/light7-swipeout-min.css","css");
					this.require("/static/libs/light7-0.4.3/js/light7-swipeout-min.js","js");
					break;
				case "fileupload"://文件上传组件
					this.require("/static/js/assets/fileupload.js","js");
					break;
				case "ajaxFileUpload"://文件异步上传组件
					this.require("/static/js/assets/ajaxFileUpload.js","js");
					this.require("/static/css/ajaxFileUpload.css","css");
					break;
				case "H5lock"://手势密码
					this.require("/static/libs/H5lock/H5lock.js","js");
					break;
				case "weiKeyBoard"://密码键盘
					this.require("/static/libs/weiKeyBoard/weiKeyBoard.js","js");
					this.require("/static/libs/weiKeyBoard/weiKeyBoard.css","css");
					break;
				case "carousel":
					this.require("/static/libs/carousel/carousel.js","js");
					break;
				case "fxapp":
					this.require("/static/js/assets/FXAPP-1.0.js","js");
					break;
                case "h5XhrFileUpload"://文件异步上传组件，支持多文件上传
					this.require("/static/js/assets/components/h5XhrFileUpload.js","js");
					this.require("/static/css/components/h5XhrFileUpload.css","css");
					break;
				case "jquery.fileupload":
					this.require("/static/libs/jquery/vendor/jquery.ui.widget.js","js");
					this.require("/static/libs/jquery/jquery.fileupload.js","js");
					break;
				case "rlsb":
					this.require("/static/rlsb/clmtrackr-dev/build/utils.js","js");
					this.require("/static/rlsb/clmtrackr-dev/build/clmtrackr.min.js","js");
					this.require("/static/rlsb/rlsb.js","js");
					break;
				default:
					try{$.alert("加载组件失败！无效的组件名("+name+")！");}catch(e){alert("加载组件失败！无效的组件名！");}
			}
		},
		/**
		 * 加载指定资源
		 * @param url
		 * @param type js或css
		 */
		require : function(url,type){
			if(type == "js"){
				document.write('<script type="text/javascript" charset="utf-8" src="'+url+'"></script>');
			}else if(type == "css"){
				document.write('<link rel="stylesheet" type="text/css" href="'+url+'"></link>');
			}
		},
		/**
		 * ui加载完之后页面初始化配置入口
		 * @param fun
		 */
		ready : function(fun){
			$(document).ready(function(){
				if(window.localStorage.getItem("appDebug") == "true"){
					new VConsole();
				}
				if(typeof fun == "function"){
					try{
						fun();
					}catch(e){
						$.alert("页面初始化失败！"+e.message);
					}
				}
				$.init();//light7初始化
			});
		},
		//以下是工具类方法定义
		/**
		 * 获取平台服务地址
		 * @returns
		 */
		getPlatformContextPath : function(useProjectName){
			var pathName=window.location.pathname;
			var projectName = pathName.substring(0,pathName.substring(1).indexOf('/')+1);
			var currentWwwPath=window.location.href;
			var position=currentWwwPath.indexOf(pathName);
			var localhostPath=currentWwwPath.substring(0,position);
			return localhostPath + (useProjectName?projectName:"/app");
		},
		/**
		 * 获取资源服务地址
		 * @returns
		 */
		getResContextPath : function(){
			var pathName=window.location.pathname;
			var currentWwwPath=window.location.href;
			var position=currentWwwPath.indexOf(pathName);
			var localhostPath=currentWwwPath.substring(0,position);
			return localhostPath + "/appres";
		},
		/**
		 * 获取平台类型(01android，02ios，03微信，04支付宝)
		 */
		getPlatformType : function(){
			var platformType = this.parameterForURL("platformType");
			if(platformType!=undefined){
				return platformType;
			}
			var deviceIsWx = navigator.userAgent.toLowerCase().match(/MicroMessenger/i) == "micromessenger";
			var deviceIsZfb = navigator.userAgent.toLowerCase().match(/AlipayClient/i) == "alipayclient";
			var deviceIsWindowsPhone = navigator.userAgent.indexOf("Windows Phone") >= 0;
			var deviceIsAndroid = navigator.userAgent.indexOf('Android') > 0 && !deviceIsWindowsPhone;
			var deviceIsIOS = /iPhone/.test(navigator.userAgent) && !deviceIsWindowsPhone;
			var miniProgram = navigator.userAgent.indexOf("miniProgram") >= 0;
			if(deviceIsWx){
				return "03";
			}else if(deviceIsZfb){
				return "04";
			}else if(deviceIsAndroid){
				return "01";
			}else if(deviceIsIOS){
				return "02";
			}else if(miniProgram){
				return "miniProgram";
			}else{
				return "01";
			}
		},
		/**
		 * 获取URL请求参数值
		 * Added By JamsonWu 2018--6-11
		 */
		getQueryString : function(name) {
			var reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
			var r = window.location.search.substr(1).match(reg);
			if (r != null) {
				return unescape(r[2]);
			}
			return null;
		},
		/**
		 * post请求
		 * @param url
		 * @param data
		 * @param fun
		 * @param waitMsg
		 * @param errorFun //Added By JamsonWu 异常自定义处理
		 */
		post : function(url, param, fun, waitMsg,errorFun,async) {
			var that = this;
			waitMsg && $.showPreloader(waitMsg);
			if (param == null) {
				param = {};
			}
			var isAsync =  true;//对默认post请求应为异步请求；对于任何同步请求应仔细考虑；同步请求一旦没有返回，会出现页面卡死情况；同时会影响页面的渲染
			if(async==false){
				isAsync = async;
			}
            //param.appcsrftoken = window.localStorage.getItem('appcsrftoken');
			// 如果是跨域请求则声明发送cookie参数到服务器
			if (url.indexOf("/") != 0 && url.indexOf(this.getPlatformContextPath() + "/") == -1) {
				$.ajax({
					type : "POST",
					xhrFields : {
						withCredentials : true
					},
					crossDomain : true,
					url : url,
					async:isAsync,
					timeout : 120000,
					data : param,
					success : function(d, textStatus, resObj) {
						var csrftoken = resObj.getResponseHeader('csrftoken');
						if (csrftoken != null && csrftoken != "") {
							window.localStorage.setItem('appcsrftoken', csrftoken);
						}
						waitMsg && $.hidePreloader();
						//统一处理异常返回
						that.handlerRes(fun, d, textStatus, resObj);
					},
					error : function(XMLHttpRequest, textStatus, errorThrown) {
						waitMsg && $.hidePreloader();
						if (errorFun) {
						  errorFun(textStatus);
						}
						else {
						  top.$.toast("请求出错！");
						}
						
					}
				});
			} else {
				/*
				 * $.post(url,data,function(d){ if(waitMsg){ $.hidePreloader(); }
				 * if(typeof(fun) == "function"){ fun(d); } });
				 */
				$.ajax({
					type : "POST",
					url : url,
					timeout : 120000,
					async:isAsync,
					data : param,
					success : function(d, textStatus, resObj) {
						var csrftoken = resObj.getResponseHeader('csrftoken');
						if (csrftoken != null && csrftoken != "") {
							window.localStorage.setItem('appcsrftoken', csrftoken);
						}
						waitMsg && $.hidePreloader();
						//统一处理异常返回
						that.handlerRes(fun, d, textStatus, resObj);
					},
					error : function(XMLHttpRequest, textStatus, errorThrown) {
						waitMsg && $.hidePreloader();
						if (errorFun) {
						   errorFun(textStatus);
						}
						else {
						  top.$.toast("请求出错！");
						}
					}
				});
			}
		},

		/**
		 * post请求
		 * @param url
		 * @param data
		 * @param fun
		 * @param waitMsg
		 * @param errorFun //Added By JamsonWu 异常自定义处理
		 */
		postEx : function(url, param, fun, waitMsg,errorFun,async) {
			//如果localhost本地开发环境并配置调用本地数据，则调用本地mock数据
			if (isGetLocalMockData(url, fun, errorFun)) {
				return;
			}
			var that = this;
			waitMsg && $.showPreloader(waitMsg);
			if (param == null) {
				param = {};
			}
			var isAsync =  true;//对默认post请求应为异步请求；对于任何同步请求应仔细考虑；同步请求一旦没有返回，会出现页面卡死情况；同时会影响页面的渲染
			if(async==false){
				isAsync = async;
			}
            //param.appcsrftoken = window.localStorage.getItem('appcsrftoken');
			// 如果是跨域请求则声明发送cookie参数到服务器
			if (url.indexOf("/") != 0 && url.indexOf(this.getPlatformContextPath() + "/") == -1) {
				$.ajax({
					type : "POST",
					xhrFields : {
						withCredentials : true
					},
					crossDomain : true,
					url : url,
					async:isAsync,
					timeout : 120000,
					data : param,
					success : function(d, textStatus, resObj) {
						var csrftoken = resObj.getResponseHeader('csrftoken');
						if (csrftoken != null && csrftoken != "") {
							window.localStorage.setItem('appcsrftoken', csrftoken);
						}
						waitMsg && $.hidePreloader();
						//统一处理异常返回
						that.handlerRes(fun, d, textStatus, resObj);
					},
					error : function(XMLHttpRequest, textStatus, errorThrown) {
						waitMsg && $.hidePreloader();
						if (errorFun) {
						  errorFun(textStatus);
						}
						else {
						  top.$.toast("请求出错！");
						}
						
					}
				});
			} else {
				/*
				 * $.post(url,data,function(d){ if(waitMsg){ $.hidePreloader(); }
				 * if(typeof(fun) == "function"){ fun(d); } });
				 */
				$.ajax({
					type : "POST",
					url : url,
					timeout : 120000,
					async:isAsync,
					data : param,
					success : function(d, textStatus, resObj) {
						var csrftoken = resObj.getResponseHeader('csrftoken');
						if (csrftoken != null && csrftoken != "") {
							window.localStorage.setItem('appcsrftoken', csrftoken);
						}
						waitMsg && $.hidePreloader();
						//统一处理异常返回
						that.handlerRes(fun, d, textStatus, resObj);
					},
					error : function(XMLHttpRequest, textStatus, errorThrown) {
						waitMsg && $.hidePreloader();
						if (errorFun) {
						   errorFun(textStatus);
						}
						else {
						  top.$.toast("请求出错！");
						}
					}
				});
			}
		},
		/**
		 * post表单请求
		 * @param url
		 * @param data
		 * @param fun
		 * @param waitMsg
		 * @param errorFun //Added By JamsonWu 异常自定义处理
		 */
		postFormData : function(url, param, fun, waitMsg,errorFun,async) {
			var that = this;
			waitMsg && $.showPreloader(waitMsg);
			if (param == null) {
				param = {};
			}
			var isAsync =  true;//对默认post请求应为异步请求；对于任何同步请求应仔细考虑；同步请求一旦没有返回，会出现页面卡死情况；同时会影响页面的渲染
			if(async==false){
				isAsync = async;
			}
            //param.appcsrftoken = window.localStorage.getItem('appcsrftoken');
			// 如果是跨域请求则声明发送cookie参数到服务器
			if (url.indexOf("/") != 0 && url.indexOf(this.getPlatformContextPath() + "/") == -1) {
				$.ajax({
					type : "POST",
					contentType: 'application/x-www-form-urlencoded',
					xhrFields : {
						withCredentials : true
					},
					crossDomain : true,
					url : url,
					async:isAsync,
					timeout : 120000,
					data : param,
					success : function(d, textStatus, resObj) {
						var csrftoken = resObj.getResponseHeader('csrftoken');
						if (csrftoken != null && csrftoken != "") {
							window.localStorage.setItem('appcsrftoken', csrftoken);
						}
						waitMsg && $.hidePreloader();
						//统一处理异常返回
						that.handlerRes(fun, d, textStatus, resObj);
					},
					error : function(XMLHttpRequest, textStatus, errorThrown) {
						waitMsg && $.hidePreloader();
						if (errorFun) {
						  errorFun(textStatus);
						}
						else {
						  top.$.toast("请求出错！");
						}
						
					}
				});
			} else {
				/*
				 * $.post(url,data,function(d){ if(waitMsg){ $.hidePreloader(); }
				 * if(typeof(fun) == "function"){ fun(d); } });
				 */
				$.ajax({
					type : "POST",
					contentType: 'application/x-www-form-urlencoded',
					url : url,
					timeout : 120000,
					async:isAsync,
					data : param,
					success : function(d, textStatus, resObj) {
						var csrftoken = resObj.getResponseHeader('csrftoken');
						if (csrftoken != null && csrftoken != "") {
							window.localStorage.setItem('appcsrftoken', csrftoken);
						}
						waitMsg && $.hidePreloader();
						//统一处理异常返回
						that.handlerRes(fun, d, textStatus, resObj);
					},
					error : function(XMLHttpRequest, textStatus, errorThrown) {
						waitMsg && $.hidePreloader();
						if (errorFun) {
						   errorFun(textStatus);
						}
						else {
						  top.$.toast("请求出错！");
						}
					}
				});
			}
		},
		/**
		 * post请求
		 * @param url
		 * @param data
		 * @param fun
		 * @param waitMsg
		 * @param errorFun //Added By JamsonWu 异常自定义处理
		 */
		jsonPost : function(url, param, fun, waitMsg,errorFun,async) {
			var that = this;
			waitMsg && $.showPreloader(waitMsg);
			if (param == null) {
				param = {};
			}
			var isAsync =  true;//对默认post请求应为异步请求；对于任何同步请求应仔细考虑；同步请求一旦没有返回，会出现页面卡死情况；同时会影响页面的渲染
			if(async==false){
				isAsync = async;
			}
			
			param.appcsrftoken = window.localStorage.getItem('appcsrftoken');
			
			// 如果是跨域请求则声明发送cookie参数到服务器
			$.ajax({
				type : "POST",
				contentType: 'application/json',
				url : url,
				timeout : 120000,
				async:isAsync,
				data : JSON.stringify(param),
				success : function(d, textStatus, resObj) {
					var csrftoken = resObj.getResponseHeader('csrftoken');
					if (csrftoken != null && csrftoken != "") {
						window.localStorage.setItem('appcsrftoken', csrftoken);
					}
					waitMsg && $.hidePreloader();
					//统一处理异常返回
					that.handlerRes(fun, d, textStatus, resObj);
				},
				error : function(XMLHttpRequest, textStatus, errorThrown) {
					waitMsg && $.hidePreloader();
					if (errorFun) {
						errorFun(textStatus);
					}
					else {
						top.$.toast("请求出错！");
					}
				}
			});
			
		},

		/**
		 * jsonp请求
		 *
		 * @param url
		 * @param data
		 * @param fun
		 * @param waitMsg
		 */
		postJsonp : function(url, data, fun, waitMsg) {
			var that = this;
			waitMsg && $.showPreloader(waitMsg);
			if (data == null) {
				data = {};
			}
			data.appcsrftoken = window.localStorage.getItem('appcsrftoken');
			// 增加callback参数
			url += (url.indexOf("?") == -1 ? "?" : "&") + "callback=?";
			for ( var i in data) {
				url += "&" + i + "=" + data[i];
			}
			if (url.indexOf("timeOut=") == -1) {
				url = url + "&timeOut=20000";
			}
			$.ajaxSetup({
				error : function() {
					waitMsg && $.hidePreloader();
					fun && fun({});
				}
			});
			$.getJSON(url, function(d, textStatus, resObj) {
				waitMsg && $.hidePreloader();
				//统一处理异常返回
				that.handlerRes(fun, d, textStatus, resObj);
			});
		},
		/**
		 * get请求，如果运行在原生app内部，则调用原生方法发请求，否则用jquery
		 * @param url
		 * @param fun
		 * @param waitMsg
		 */
		get : function(url, fun, waitMsg) {
			var that = this;
			waitMsg && $.showPreloader(waitMsg);
			if (typeof (FXAPP) != "undefined" && FXAPP.isApp() && typeof (FXAPP.post) == "function") {
				var param = {};
				param.url = url;
				var fail = function(errMsg) {
					waitMsg && $.hidePreloader();
					$.alert(errMsg);
				}
				var success = function(d) {
					waitMsg && $.hidePreloader();
					fun && fun(d);
				}
				FXAPP.get(param, success, fail);
			} else {
				$.get(url, function(d, textStatus, resObj) {
					waitMsg && $.hidePreloader();
					//统一处理异常返回
					that.handlerRes(fun, d, textStatus, resObj);
				});
			}
		},
		//统一处理返回结果
		handlerRes : function(fun, data, textStatus, resObj){
			var errId = resObj ? resObj.getResponseHeader('EXCEPTION_CODE') : undefined;
			if (errId) {
				this.error({ errId : errId, errMsg : '系统内部异常！' });
			} else {
				fun && fun(data);
			}
		},
		error : function(data) {
			$.alert(data.errMsg);
		},
		/**
		 * 检查是否允许跳转到指定url
		 */
        checkRedirectUrl : function (url) {
            var checkRedirectUrl = false;
            if(url){
                var loca = window.location.href;
                loca = loca.substring(loca.indexOf("//")+2);
                loca = loca.substring(0,loca.indexOf("/"));
                checkRedirectUrl = url.indexOf(loca) != -1;
            }
			return checkRedirectUrl;
		},
		/**
		 * 从url中获取指定参数
		 */
		parameterForURL : function (name) {
			var url = window.location.href;
			var p = -1;
			var value = undefined;
			if (((p = Math.max(url.indexOf("&" + name + "="), url.indexOf("?" + name
					+ "="))) > -1)) {
				value = url.substring(p + name.length + 2);
				if (value.indexOf("&") > -1) {
					value = value.substring(0, value.indexOf("&"));
				} else if (value.indexOf("#") > -1) {
					value = value.substring(0, value.indexOf("#"));
				}
			}
			if ((new RegExp("[`~!@#$^&*()\\-\\+=|{}':;',\\[\\]<>/?~！@#￥……&*（）—|{}【】‘；：”“'。，、？]")).test(value)) {
				return undefined;
			}
			return value;
		},
		/**
		 * 从localStorage和url中获取参数值
		 */
		parameterForLSAndURL : function (LSkey,URLname) {
			var value = undefined;
			if(value===""||value==null||typeof(value)==="undefined"){
				if(URLname!=null&&URLname!=""){
					value = this.parameterForURL(URLname);
				}else{
					value = undefined;
				}
			}
			if(LSkey!=null && typeof(value) == "undefined"){
                value = localStorage.getItem(LSkey);
			}
			return value==undefined?null:value;
		},
		/**
		 * 增加页脚
		 */
		addFooter : function(){
			var footer = '<nav class="bar bar-tab" style="font-size:0.6rem;">'
						+'<table width="100%" height="100%" border="0" cellspacing="0" cellpadding="0"><tr><td align="right"><img src="/static/images/common/fxlogo.png" style="margin:0.2rem;width:1rem;display:block;"/></td><td align="left">方欣移动办税&nbsp;-&nbsp;客服电话：020-62847100</td></tr></table>'
						+'</nav>';
			//$(document.body).prepend(footer);
		},
        /**
		 * 跳到指定pageIndex
         */
		jumpToPageIndex:function () {
            var fxAppId = window.localStorage.getItem("FxAppId");
            var tkey = fxAppId+'_pageIndex';
            var pageIndex = window.localStorage.getItem(tkey);
            if(document.referrer.indexOf("index.html") != -1 && document.referrer.indexOf("FxAppId") != -1){
            	console.log("index")
            }else{
            	if(pageIndex) {
                    $.router.loadPage("#"+pageIndex);
                    window.localStorage.removeItem(tkey);
    			}else {
                	console.log('no pageIndex');
    			}
            }
        },
		/**
		 * 跳转到指定页面
		 * @param page
		 */
		goPage : function (page,action){
			var that = this;
			$.showIndicator();
		    setTimeout(function () {
		    	if(action!=null&&action=="replace"){
		    		window.location.replace(page);
		    	}else{
			    	try{
						var turlArray = window.location.href.split('#');
						if(turlArray.length == 2) {
							var fxAppId = window.localStorage.getItem("FxAppId");
							if(fxAppId) {
								window.localStorage.setItem(fxAppId+'_pageIndex',turlArray[1]);
	                        }
						}
					}catch (e)  {console.log(e)}
			        //申报征收前端开发环境增加端口配置 added by wuchangsen 2019-01-11
					var debugMode = sessionStorage.getItem("debugMode")
					var devPort = sessionStorage.getItem("devPort")
					if (debugMode && devPort && page && page.indexOf("http") < 0 && page.indexOf("#") >= 0) {
				  	   window.location.href = location.protocol + "//" + location.host +":" + devPort + page;
					}
					else {
					  window.location.href = page;
					}
		    	}
		    }, 100);
		    setTimeout(function () {
		        $.hideIndicator();
		    }, 100);
		},
		/**
		 * 跳转到指定页面（如果原页面有指定返回页面，则跳到该返回页面）
		 * @param page
		 */
		goPageIfRedirect : function (page,redirectParam){
			$.showIndicator();
            if(!redirectParam)redirectParam='redirectUrl';
            var redirectUrl = this.parameterForURL(redirectParam);
		    setTimeout(function () {
		    	if(redirectUrl) {
                    top.location.href = decodeURIComponent(redirectUrl);
				}else {
		        	window.location.href = page;
				}
		    }, 100);
		    setTimeout(function () {
		        $.hideIndicator();
		    }, 3000);
		},
		/**
		 * 跳转到指定页面(如果页面有指定返回页面，则将返回页面url附带到下一页面）
		 * @param page
		 */
		goPageWithRedirect : function (page,redirectParam){
			$.showIndicator();
			if(!redirectParam)redirectParam='redirectUrl';
			var redirectUrl = this.parameterForURL(redirectParam);
		    setTimeout(function () {
		    	if(page.indexOf("?") >= 0 && page.indexOf(redirectParam+"=") <0 && redirectUrl) {
		    		page += "&" + redirectParam + "=" + redirectUrl;
				}else if(page.indexOf("?") < 0 && page.indexOf(redirectParam+"=") <0 && redirectUrl) {
                    page += "?" + redirectParam + "=" + redirectUrl;
				}
		        window.location.href = page;
		    }, 100);
		    setTimeout(function () {
		        $.hideIndicator();
		    }, 3000);
		},
		pushAllSessions : function(fun, doInBackground) {
			!doInBackground && $.showIndicator();
			var that = this;
			var params = {
				FxAppId : window.localStorage.FxAppId||window.localStorage.getItem("FxAppId")||this.parameterForURL("FxAppId"),
				appId : window.localStorage.FxAppId||window.localStorage.getItem("FxAppId")||this.parameterForURL("FxAppId"),
				platformType : this.getPlatformType()
			};
			this.post(this.getPlatformContextPath() + "/index/getAllServers.do", params, function(d) {
				if (d.servers && d.servers.length > 0) {
					var pushSession = function(servers) {
						var server = servers.pop();
						if (servers.length == 0) {
							that.pushSessionToBizfront(fun, server.serverId, server.serverType);
						} else {
							that.pushSessionToBizfront(function() {
								pushSession(servers);
							}, server.serverId, server.serverType);
						}
					};
					pushSession(d.servers);
				} else {
					fun && fun();
				}
			});
		},
		//以下是业务相关方法
		/**
		 * 把当前用户会话信息推送到局端计入点
		 * @param fun 推送完成的回调函数
		 * @param serverId 接入点id，如果不传则默认推送到当前应用的默认接入点
		 * @param gdslx 需要推送到接入点的国税服务还是地税服务
		 * @param waitMsg
		 */
		pushSessionToBizfront : function(fun,serverId,gdslx,waitMsg){
			var that = this;
			this.post(this.getPlatformContextPath()+"/user/getServerUrlAndUserMd5.do",{serverId:serverId,gdslx:gdslx}, function(d){
				if(d.flag == "error"){
					fun && fun(false);
				}else{
					var userMd5 = d.userMd5;
					var serverUrl = d.serverUrl;
					//请求局端接入点检查是否需要推送会话
					that.postJsonp(serverUrl+"/session/checkSession.do",{userMd5:userMd5},function(d){
						var redisSessionCookieName = d.redisSessionCookieName;
						var appfrontSessionId = d.appfrontSessionId;
						if(d.flag == "setSession"){//需要推送会话
							that.post(that.getPlatformContextPath()+"/user/setAppfrontSession.do", {serverId:serverId,appfrontSessionId:appfrontSessionId,gdslx:gdslx}, function(d){
								fun && fun(true,serverId,redisSessionCookieName,appfrontSessionId);
							}, waitMsg);
						}else{
							fun && fun(false,serverId);
						}
					});
				}
			}, waitMsg)
		},
		/**
		 * 检查用户是否有登录
		 * @param isGotoLoginPage 没登录的时候是否要跳转到登录页
         * @param redirectUrl 登录后重定向地址
		 * @param fun 回调
		 */
		checkIsLogin : function(isGotoLoginPage, redirectUrl, fun){
			var that = this;
			if (typeof(redirectUrl) == 'function') {
			    fun = redirectUrl;
                redirectUrl = '';
            }
			this.post(this.getPlatformContextPath()+"/user/getAppUserDetails.do",{},function(d){
				if(d.userDetails == null && isGotoLoginPage){
					$.toast("请先登录！");
					var backUrl = that.getPlatformContextPath()+"/assets/user/login.html";
					if (redirectUrl) {
                        backUrl += "?redirectUrl=" + redirectUrl;
                    }
					that.goPage(backUrl);
				}else{
					if(typeof(fun) == "function"){
						fun(d.userDetails,d.isUserChange,d.currentApp,d.currentServer,d);
					}
				}
			},"请稍候...");
		},
		/**
		 * 把当前页面的title设置为应用名称
		 */
		setPageTitle : function() {
			var that = this;
			$(document).ready(function() {
				var aikey = (that.parameterForURL("FxAppId") || window.localStorage.FxAppId) + '_appInfo';
				if (window.localStorage[aikey]) {
					var appInfo = JSON.parse(window.localStorage[aikey]);
					$("title").html(appInfo.app.name);
				}
			});
		},
        /**
         * 微信配置设置
         */
        wxConfig:function (wxReady) {
        	var _this = this;
            //初始化微信jssdk配置
            var localUrl = window.location.href.split('#')[0];
            var param = {url:localUrl};
            this.post(this.getPlatformContextPath()+"/wx/getJsSdkConfig.do",param,function(d){
                wx.config({
                	beta: true,//注入wx.invoke 和 wx.on 方法。wx.invoke 用于调用未开放的由页面主动调用的 JS 接口，wx.on 用于调用未开放的监听类 JS 接口。
                    debug : false, // 开启调试模式,调用的所有api的返回值会在客户端alert出来，若要查看传入的参数，可以在pc端打开，参数信息会通过log打出，仅在pc端时才会打印。
                    appId : d.appId, // 必填，公众号的唯一标识
                    timestamp : d.timestamp, // 必填，生成签名的时间戳
                    nonceStr : d.nonceStr, // 必填，生成签名的随机串
                    signature : d.signature,// 必填，签名，见附录1
                    jsApiList : ["onMenuShareTimeline", "onMenuShareAppMessage",
                        "onMenuShareQQ", "onMenuShareWeibo", "startRecord",
                        "stopRecord", "onVoiceRecordEnd", "playVoice",
                        "pauseVoice", "stopVoice", "onVoicePlayEnd",
                        "uploadVoice", "downloadVoice", "chooseImage",
                        "previewImage", "uploadImage", "downloadImage",
                        "translateVoice", "getNetworkType", "openLocation",
                        "getLocation", "hideOptionMenu", "showOptionMenu",
                        "hideMenuItems", "showMenuItems",
                        "hideAllNonBaseMenuItem", "showAllNonBaseMenuItem",
                        "closeWindow", "scanQRCode", "chooseWXPay",
                        "openProductSpecificView", "addCard", "chooseCard",
                        "openCard","requestWxFacePictureVerify","checkIsSupportFaceDetect"]// 必填，需要使用的JS接口列表，所有JS接口列表见附录2
                });
                wx.ready(function() {
                	_this.IsWxConfig = true;
                    if (typeof (wxReady) == "function") {
                        wxReady();
                    }
                });
            });
        },
		scanQRcode:function (params, success, fail) {
            //若要使用FXAPP 请在页面上加入FXUI.requireComponent("fxapp");
        	if(FXAPP && FXAPP.isApp()) {
                FXAPP.scanQRCode(params, success, fail);
			}else {
        		var func = function () {
                    wx.scanQRCode({
                        needResult: 1, // 默认为0，扫描结果由微信处理，1则直接返回扫描结果，
                        scanType: ["qrCode","barCode"], // 可以指定扫二维码还是一维码，默认二者都有
                        success: function (res) {
                            var result = res.resultStr; // 当needResult 为 1 时，扫码返回的结果
                            if(typeof (success) == "function") {
                                success(result);
                            }
                        }
                    });
                }
        		if(!this.IsWxConfig) {
        			this.wxConfig(func);
				}else {
        			func();
				}

			}
        },
        /**
         * 敏感信息字段，部分字符用*代替的方法
         * type:"sjhm"-手机号码
         *      "xm"-姓名
         *      "zjhm"-证件号码
         *      "other"-其他的规则（包括识别号、其他证件类型）
         */
        hideSecretContent:function(number,type){
        	if(number != null && number != ""){
        		var x = number.length;
        		var a,b,c;
        		if(type == "sjhm"){
        			a = 3;
        			c = 4;
        			b = x - a - c;
        		} else if(type == "xm"){
        			a = 0;
        			c = 1;
        			b = x - a - c;
        		} else if(type == "zjhm"){
					// a = 6;
					a = 4;
        			c = 4;
        			b = x - a - c;
        		}else if(type == "other"){
        			if(x%4 == 0){
        				a = x/4;
        				b = x/2;
        				c = x/4;
        			}else if(x%2 == 0 && x%4 != 0){
        				a = parseInt(x/4);
        				b = x/2;
        				c = parseInt(x/4)+1;
        			}else if(x%2 != 0){
        				a = parseInt((x-parseInt(x/2))/2);
        				b = parseInt(x/2);
        				c = x-parseInt(x/2)-parseInt((x-parseInt(x/2))/2);
        			}
        		}
        		var mod = "";
        		for(var k=0;k<b;k++){
        			mod = mod + "*";
        		}
        		return number.substring(0,a)+mod+number.substring(x-c,x);
        	}else{
        		return number;
        	}
        },
        setIndexTab : function(i) {
        		var templateId = window.localStorage.templateId;
        		if (!templateId) {
        			return;
        		}
        		window.localStorage[templateId + '_selectedIndexKey'] = i;
        },
        isWx : function() {
        		return this.getPlatformType() === '03';
        },
    	/*  去除苹果手机的页面返回按钮
    	 * （云南地税有这样的需求控制，若其他地方地税不需要这样的控制，可去掉该方法）
    	 */
        setBackBtn:function(){
    		$(document).ready(function(){
    			var deviceIsWx = navigator.userAgent.toLowerCase().match(/MicroMessenger/i) == "micromessenger";
    			if(deviceIsWx){  //微信的
    				var ua = navigator.userAgent.toLowerCase();
    				if(ua.match(/iPhone/i)=="iphone") {
    					var spans = document.getElementsByTagName("span");
    					for(var i=0;i<spans.length;i++){
    						var span = spans[i];
    						var $span = $(span);
    						if($span.hasClass("icon-left")){
    							var a = $span.parent();
    							if($(a).hasClass("back")){
    								//使用back返回，直接去掉返回
    								$span.removeClass("icon-left");
    								$(a).removeClass("back");
    							} else{
    								//自定义返回，去掉返回2字
    								$span.removeClass("icon-left");
    								$span.addClass("icon-left2");
    							}
    						}
    					}
    				}
    			}
    		});
    	},
    	/*  去除支付宝城市服务的页面返回按钮
    	 * （云南地税有这样的需求控制，若其他地方地税不需要这样的控制，可去掉该方法）
    	 */
    	setAliAppBackBtn:function(){
    		$(document).ready(function(){
    			var deviceIsWx = navigator.userAgent.toLowerCase().match(/AliApp/i) == "aliapp";
    			if(deviceIsWx){
    				var headers = document.getElementsByTagName("header");
    				for(var i=0;i<headers.length;i++){
    					var header = headers[i];
    					$(header).attr("style","display: none;");
    					var page =header.parentNode;
    					var childrens=page.children;
    					for(var j=0;j<childrens.length;j++){
    						var children = childrens[j];
    					    if($(children).attr("class")=="content"){
    					    	$(children).attr("style","top:0;")
    					    }
    					}
    				}
    				/*var spans = document.getElementsByTagName("span");
					for(var i=0;i<spans.length;i++){
						var span = spans[i];
						var $span = $(span);
						if($span.hasClass("icon-left")){
							var a = $span.parent();
							if($(a).hasClass("back")){
								//使用back返回，直接去掉返回
								$span.removeClass("icon-left");
								$(a).removeClass("back");
							} else{
								//自定义返回，去掉返回2字
								$span.removeClass("icon-left");
								$span.addClass("icon-left2");
							}
						}
					}*/
    			}
    		});
		}
	}
	window.FXUI = new FXUI();
	// window.
})($);

$(function ($) {
	// 根据url参数bspj的值显示、隐藏评价按钮
	function canselHcpBtn() {
		var btn = document.getElementById('hcpBtn')
		if (!btn) return
		var param = HCP_getBspjParam()
		param === 'Y' ? $('#hcpBtn').show() : $('#hcpBtn').hide()
	}
	
	// 获取url的bspj参数
	window.HCP_getBspjParam = function () {
		var url = window.location.search.substring(1)
		var params = url.split('&')
		var target = ''
		params.some(function (item) {
			if (item.indexOf('bspj') != -1) {
				target = item.toLowerCase().indexOf('y') != -1 ? 'Y' : 'N'
				return true
			}
		})
		return target ? target : 'Y'
	}
	
	function checkPjUser(callBack) {
		FXUI.post("/app/user/getAppUserDetails.do", {}, function (res) {
			if (!res.userDetails) {
				$.alert('请登录后重试！')
			} else if (!res.userDetails.zrr || !res.userDetails.zrr.zjhm) {
				$.alert('当前用户未实名，请先实名再操作！')
			} else {
				callBack()
			}
		}, '加载中...');
	}
	
	/**
	 * 校验登录状态、用户身份
	 * noLogin为true时，表示用户不登录评价，需要构建评价的用户，pjUser不能为空
	 */
	window.HCP_checkUserState = function (noLogin, pjUser, callBack) {
		if (typeof noLogin == 'function') {
			checkPjUser(noLogin);
			return;
		}
		if (noLogin) {
			if (pjUser == null || pjUser == {}) {
				$.alert('暂无用户信息');
				return;
			}
			FXUI.jsonPost("/app/pjxx/buildPjUser.do", pjUser, function (res) {
				if (res.errcode == 'error') {
					$.alert(res.errmsg);
					return;
				}
				callBack();
			})
		} else {
			checkPjUser(callBack)
		}
	}
	
	function HCP() {}
	HCP.prototype.gotoHCP = function (ywlx, ywbm, noLogin, pjUser) {
		this.checkUserState(function () {
			window.location.href = '/mtax/components/hcp/PageSingleEvaluate?ywlx='+ywlx+'&ywbm='+ywbm;
		}, noLogin, pjUser)
	}
	/**
	 * 校验登录状态、用户身份
	 * noLogin为true时，表示用户不登录评价，需要构建评价的用户，pjUser不能为空
	 */
	HCP.prototype.checkUserState = function (callBack, noLogin, pjUser) {
		if (noLogin === true) {
			if (pjUser == null || pjUser == {}) {
				$.toast('暂无用户信息');
				return;
			}
			FXUI.jsonPost("/app/pjxx/buildPjUser.do", pjUser, function (res) {
				if (res.errcode == 'error') {
					$.toast(res.errmsg);
					return;
				}
				callBack();
			})
		} else {
			//需要登录用户身份
			FXUI.post("/app/user/getAppUserDetails.do", {}, function (res) {
				if (!res.userDetails) {
					$.toast('请登录后重试！');
				} else if (!res.userDetails.zrr || !res.userDetails.zrr.zjhm) {
					$.toast('当前用户未实名，请先实名再操作！');
				} else {
					callBack()
				}
			}, '加载中...');
		}
	}
	/**
	 * 好差评按钮显示状态和形状
	 * bspj=0（不显示）、1（圆形按钮，默认）、其他（矩形按钮）
     */
    HCP.prototype.showAndDesign = function () {
        var bspj = FXUI.getQueryString("bspj");
        
        var hcpBtns = $('[id="hcpBtn"]');
        if (bspj == '0') {
            hcpBtns.hide();
        } else if (!bspj || bspj == '1') {
            //默认圆形按钮
            hcpBtns.removeAttr('style').removeAttr('class');
            hcpBtns.children('button').removeAttr('style').removeAttr('class');
            hcpBtns.addClass('roundBtn');
        }
    }
	window.HCP = new HCP();
	window.HCP.showAndDesign();
})

FXUI.ready(function() {
	function loadScript(script, _document, callback) {
		if (!script) {
			return;
		}
		if (Object.prototype.toString.call(script) !== '[object Array]') {
			script = [ script ];
		}
		if (typeof _document === 'undefined') {
			_document = document;
		}
		if (typeof _document === 'function') {
			callback = _document;
			_document = document;
		}
		var index = 0;
		var appendScript = function() {
			if (index >= script.length) {
				if (typeof callback === 'function') {
					setTimeout(function() {
						callback();
					}, 500);
				}
				return;
			}
			var url = script[index++];
			if (!url || _document.querySelector('script[src="' + url + '"]')) {
				appendScript();
				return;
			}
			var _script = _document.createElement('script');
			_script.type = 'text/javascript';
			_script.src = url;
			appendToHead(_document, _script);
			if (_script.readyState) {
				_script.onreadystatechange = function() {
					if (_script.readyState === 'loaded'
							|| _script.readyState === 'complete') {
						_script.onreadystatechange = null;
						appendScript();
					}
				}
			} else {
				_script.onload = function() {
					appendScript();
				}
				_script.onerror = function() {
					appendScript();
				}
			}
		};
		appendScript();
	}

	function appendToHead(_document, element) {
		var head = _document.getElementsByTagName('head');
		(head && head.length > 0) && head[0].appendChild(element);
	}

	var $header = $('.page>.bar-nav');
	if ($header[0]) {
		var u = navigator.userAgent;
		var isiOS = FXUI.getPlatformType() == '02';
		var isAndroid = FXUI.getPlatformType() == '01';
		if ((isiOS && window.webkit) || (isAndroid && typeof android != 'undefined')) {
			loadScript('/static/js/assets/QinyibanJssdk1.0.js', function() {
				$('.pull-left', $header).unbind('click').click(function() {
					var keys = '关闭';
					if (isiOS) {
						window.webkit.messageHandlers.PopToApp.postMessage(keys);
					} else if (isAndroid) {
						android.PopToApp(keys);
					}
				});
			});
		}
	}
});
