(function() {
	'use strict';
	var $, Banner, BannerItem;

	$ = jQuery;

	Banner = (function() {

		Banner.description = {
			name : 'Banner',
			desc : '横幅广告',
			icon : ''
		};

		function Banner(widget, parent) {
			Widget.apply(this, arguments);

			this.settings = this.widget.settings || {};
			this.bannerItems = [];
			this.images = widget.images;
			this.width = this.window().width;
			this.height = this.settings.height || 160;
			this.interval = this.settings.interval || 3000;

			this.$widget = $('<div class="banner"></div>').attr('id', this.id);
			this.$carousel = $('<div class="carousel"></div>');
			this.$indicators = $('<ol class="carousel-indicators"></ol>');
			this.$inner = $('<div class="carousel-inner"></div>');
			this.$carousel.append(this.$indicators).append(this.$inner);
			this.$widget.append(this.$carousel);

			return this;
		}

		Banner.prototype.init = function() {
			this.parent.$widget.append(this.$widget);
			this.loadSetting();
			for (var i = 0, j = this.images.length; i < j; i++) {
				var bannerItem = new BannerItem(this.images[i], this);
				this.$indicators.append(bannerItem.$indicator);
				this.$inner.append(bannerItem.$bannerItem);
				this.bannerItems.push(bannerItem);
			}
			var a = this;
			this.carousel = this.$inner.carousel({
				wrap : false,
				vertical : false,
				horizontal : true,
				pagingFunction : function(index, item) {
					a.select(a.bannerItems[index]);
				}
			});
			this.showOrHideIndicator();
			this.resetIndex();
			this.play();
			this.select(this.bannerItems[0]);
			return this;
		};

		Banner.prototype.loadSetting = function() {
			if (/^(100|[1-9]\d|\d)(.\d{1,2})?%$/.test(this.height)) {
				this.height = this.window().height * parseInt(this.height) / 100;
			}
			this.$widget.css('height', this.height);
			this.$widget.css('width', this.width);
			return this;
		};

		Banner.prototype.exportWidget = function() {
			this.widget.widgets = [];
			this.widget.images = [];
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				this.widget.images.push(this.bannerItems[i].image);
			}
			this.widget.settings = this.settings;
			return this.widget;
		};

		Banner.prototype.add = function(bannerItem) {
			this.$indicators.append(bannerItem.$indicator);
			this.$inner.append(bannerItem.$bannerItem);
			this.bannerItems.push(bannerItem);
			this.resetIndex();
			this.showOrHideIndicator();
		};

		Banner.prototype.remove = function(bannerItem) {
			var position = 0;
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				if (bannerItem.id == this.bannerItems[i].id) {
					position = i;
					break;
				}
			}
			this.bannerItems.splice(position, 1);
			this.resetIndex();
			if (bannerItem.active) {
				this.currentBannerItem = null;
				this.select(this.bannerItems[0]);
			}
			this.showOrHideIndicator();
		};

		Banner.prototype.showOrHideIndicator = function() {
			this.bannerItems.length <= 1 ? this.$indicators.hide() : this.$indicators.show();
		};

		Banner.prototype.resetIndex = function() {
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				this.bannerItems[i].index = i;
			}
		};

		Banner.prototype.getCurrentIndex = function() {
			return this.currentBannerItem ? this.currentBannerItem.index : 0;
		};

		Banner.prototype.getBannerItemByIndex = function(index) {
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				if (this.bannerItems[i].index === index) {
					return this.bannerItems[i];
				}
			}
		};

		Banner.prototype.play = function() {
			if (this.isPlaying) {
				return;
			}
			this.isPlaying = true;
			this.restartTimer();
		};

		Banner.prototype.pause = function() {
			if (this.noPause) {
				return;
			}
			this.isPlaying = false;
			this.resetTimer();
		};

		Banner.prototype.next = function() {
			var index = (this.getCurrentIndex() + 1) % this.bannerItems.length;
			this.select(this.getBannerItemByIndex(index));
		};

		Banner.prototype.select = function(bannerItem) {
			if (bannerItem && bannerItem !== this.currentBannerItem) {
				this.goNext(bannerItem);
			}
		};

		Banner.prototype.goNext = function(bannerItem) {
			bannerItem.active = true;
			if (this.currentBannerItem) {
				this.currentBannerItem.active = false;
			}
			this.currentBannerItem = bannerItem;
			this.carousel.onMoveIndex(this.currentBannerItem.index);
			this.restartTimer();
		};

		Banner.prototype.restartTimer = function() {
			var a = this;
			this.resetTimer();
			var t = +this.interval;
			if (!isNaN(t) && t > 0) {
				this.currentInterval = setInterval(function() {
					var t = +a.interval;
					a.isPlaying && !isNaN(a.interval) && t > 0
							&& a.bannerItems.length > 0 ? a.next() : a.pause();
				}, t);
			}
		};

		Banner.prototype.resetTimer = function() {
			if (this.currentInterval) {
				clearInterval(this.currentInterval);
				this.currentInterval = null;
			}
		};

		Banner.prototype.setLogo = function() {
			if (!this.widget.logo) {
				return;
			}
			if (!this.$logo) {
				this.$logo = $('<div class="carousel-logo"></div>');
				this.$carousel.append(this.$logo);
			}
			var a = this;
			this.$logo.css({
				width: a.width * 0.8,
				height: a.height / 3,
				'margin-left': -(a.width * 0.8 * 0.5),
				'margin-top': -(a.height / 3 * 0.5)
			});
			var $img = $('<img/>').attr('src', this.widget.logo);
			this.$logo.empty().append($img);
		};

		Banner.prototype.show = function() {
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				this.bannerItems[i].show();
			}
			this.setLogo();
		};

		Banner.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.height
			};
		};

		Banner.prototype.resize = function() {
			this.width = this.window().width;
			this.loadSetting();
		};

		return Banner;

	})();

	BannerItem = (function() {

		function BannerItem(image, parent) {
			this.id = 'BannerItem' + (parent.index++);
			this.image = image;
			this.parent = parent;
			this.index = -1;
			this.active = false;

			this.link = __bind(this.link, this);
			this.select = __bind(this.select, this);

			this.$indicator = $('<li></li>');
			this.$bannerItem = $('<div></div>').addClass('carousel-item active');

			this.$indicator.click(this.select);
			this.watch('active', this.activeChange);

			return this;
		}

		BannerItem.prototype.watch = function(property, callback) {
			if (property in this) {
				var old = this[property];
				Object.defineProperty(this, property, {
					set : function(value) {
						var o = old;
						old = value;
						callback(value, o, this);
					},
					get : function() {
						return old;
					}
				});
			}
		};

		BannerItem.prototype.activeChange = function(active, old, obj) {
			if (active) {
				obj.$indicator.addClass('active');
			} else {
				obj.$indicator.removeClass('active');
			}
		};

		BannerItem.prototype.link = function() {
			if (this.image.url) {
				FXUI.goPage(this.image.url);
			}
		};

		BannerItem.prototype.select = function() {
			this.parent.select(this);
		};

		BannerItem.prototype.show = function() {
			if (!this.$image) {
				this.$image = $('<img />').attr('src', this.image.src).attr('onerror', 'this.src="/appres/file/2019/07/23/1563846685071.png"');;
				this.$bannerItem.append(this.$image);
				this.$image.click(this.link);
			}
		};

		return BannerItem;

	})();

	Widget.register('Banner', Banner);

}).call(this);