(function() {
	'use strict';
	var $, Banner2, BannerItem;

	$ = jQuery;

	Banner2 = (function() {

		Banner2.description = {
			name : 'Banner2',
			desc : '总局横幅广告',
			icon :  ''
		};

		function Banner2(widget, parent) {
			Widget.apply(this, arguments);

			this.settings = this.widget.settings || {};
			this.bannerItems = [];
			this.images = widget.images;
			this.width = this.window().width;
			this.height = this.settings.height || 160;
			this.interval = this.settings.interval || 3000;

			this.$widget = $('<div class="banner"></div>').attr('id', this.id);
			this.$carousel = $('<div class="carousel"></div>');
			this.$indicators = $('<ol class="carousel-indicators2"></ol>');
			this.$inner = $('<div class="carousel-inner"></div>');
			this.$carousel.append(this.$indicators).append(this.$inner);
			this.$widget.append(this.$carousel);

			return this;
		}

		Banner2.prototype.init = function() {
			this.parent.$widget.append(this.$widget);
			this.loadSetting();
			for (var i = 0, j = this.images.length; i < j; i++) {
				var bannerItem = new BannerItem(this.images[i], this);
				this.$indicators.append(bannerItem.$indicator);
				this.$inner.append(bannerItem.$bannerItem);
				this.bannerItems.push(bannerItem);
			}
			var a = this;
			this.carousel = this.$inner.carousel({
				wrap : false,
				vertical : false,
				horizontal : true,
				pagingFunction : function(index, item) {
					a.select(a.bannerItems[index]);
				}
			});
			this.resetIndex();
			this.play();
			this.select(this.bannerItems[0]);
			return this;
		};

		Banner2.prototype.loadSetting = function() {
			if (/^(100|[1-9]\d|\d)(.\d{1,2})?%$/.test(this.height)) {
				this.height = this.window().height * parseInt(this.height) / 100;
			}
			this.$carousel.height(this.height);
			this.$widget.height(this.height);
			this.$widget.width(this.width);
			
			return this;
		};

		Banner2.prototype.exportWidget = function() {
			this.widget.widgets = [];
			this.widget.images = [];
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				this.widget.images.push(this.bannerItems[i].image);
			}
			this.widget.settings = this.settings;
			return this.widget;
		};

		Banner2.prototype.resetIndex = function() {
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				this.bannerItems[i].index = i;
			}
		};

		Banner2.prototype.getCurrentIndex = function() {
			return this.currentBannerItem ? this.currentBannerItem.index : 0;
		};

		Banner2.prototype.getBannerItemByIndex = function(index) {
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				if (this.bannerItems[i].index === index) {
					return this.bannerItems[i];
				}
			}
		};

		Banner2.prototype.play = function() {
			if (this.isPlaying) {
				return;
			}
			this.isPlaying = true;
			this.restartTimer();
		};

		Banner2.prototype.pause = function() {
			if (this.noPause) {
				return;
			}
			this.isPlaying = false;
			this.resetTimer();
		};

		Banner2.prototype.next = function() {
			var index = (this.getCurrentIndex() + 1) % this.bannerItems.length;
			this.select(this.getBannerItemByIndex(index));
		};

		Banner2.prototype.select = function(bannerItem) {
			if (bannerItem && bannerItem !== this.currentBannerItem) {
				this.goNext(bannerItem);
			}
		};

		Banner2.prototype.goNext = function(bannerItem) {
			bannerItem.active = true;
			if (this.currentBannerItem) {
				this.currentBannerItem.active = false;
			}
			this.currentBannerItem = bannerItem;
			this.carousel.onMoveIndex(this.currentBannerItem.index);
			this.restartTimer();
		};

		Banner2.prototype.restartTimer = function() {
			var a = this;
			this.resetTimer();
			var t = +this.interval;
			if (!isNaN(t) && t > 0) {
				this.currentInterval = setInterval(function() {
					var t = +a.interval;
					a.isPlaying && !isNaN(a.interval) && t > 0
							&& a.bannerItems.length > 0 ? a.next() : a.pause();
				}, t);
			}
		};

		Banner2.prototype.resetTimer = function() {
			if (this.currentInterval) {
				clearInterval(this.currentInterval);
				this.currentInterval = null;
			}
		};

		Banner2.prototype.show = function() {
			for (var i = 0, j = this.bannerItems.length; i < j; i++) {
				this.bannerItems[i].show();
			}
		};

		Banner2.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.height
			};
		};

		Banner2.prototype.resize = function() {
			this.width = this.window().width;
			this.loadSetting();
		};

		return Banner2;

	})();

	BannerItem = (function() {

		function BannerItem(image, parent) {
			this.id = 'BannerItem' + (parent.index++);
			this.image = image;
			this.parent = parent;
			this.index = -1;
			this.active = false;

			this.link = __bind(this.link, this);
			this.select = __bind(this.select, this);

			this.$indicator = $('<li><span class="icon2-indicator"></span></li>');
			this.$bannerItem = $('<div></div>').addClass('carousel-item active');

			this.$indicator.click(this.select);
			this.watch('active', this.activeChange);

			return this;
		}

		BannerItem.prototype.watch = function(property, callback) {
			if (property in this) {
				var old = this[property];
				Object.defineProperty(this, property, {
					set : function(value) {
						var o = old;
						old = value;
						callback(value, o, this);
					},
					get : function() {
						return old;
					}
				});
			}
		};

		BannerItem.prototype.activeChange = function(active, old, obj) {
			if (active) {
				obj.$indicator.addClass('active');
			} else {
				obj.$indicator.removeClass('active');
			}
		};

		BannerItem.prototype.link = function() {
			if (this.image.url) {
				FXUI.goPage(this.image.url);
			}
		};

		BannerItem.prototype.select = function() {
			this.parent.select(this);
		};

		BannerItem.prototype.show = function() {
			if (!this.$image) {
				this.$image = $('<img />').attr('src', this.image.src).attr('onerror', 'this.src="/appres/file/2019/07/23/1563846685071.png"');
				this.$bannerItem.append(this.$image);
				this.$image.click(this.link);
			}
		};

		return BannerItem;

	})();

	Widget.register('Banner2', Banner2);

}).call(this);