(function() {
	'use strict';
	var $, Folder;

	$ = jQuery;

	Folder = (function() {

		Folder.description = {
			name : 'Folder',
			desc : '文件夹',
			icon : ''
		};

		function Folder(widget, parent) {
			GridItem.apply(this, arguments);

			this.normalWidth = 56;
			this.dropWidth = 66;
			this.subWidgets = [];
			this._$widget = this.$widget;
			this.$icon.empty();
			this.$icon.addClass('folder_item_icon');
			this.folderPagingId = this.id + 'Paging';
			this.$hTitle = $('<input type="text" />').val(this.widget.title);
			this.$header = $('<div></div>').addClass('folder_header').append(this.$hTitle);
			this.$widget = $('<div></div>').addClass('folder_body');
			this.$wrapper = $('<div></div>').addClass('folder_wrapper').append(this.$header).append(this.$widget);
			this.$folderContainer = $('#folder_container');

			return this;
		}

		for ( var i in GridItem.prototype) {
			Folder.prototype[i] = GridItem.prototype[i];
		}
		Folder.prototype.constructor = Folder;

		Folder.prototype.init = function() {
			var a = this;
			if (a.parent.row && a.parent.subWidgets.length >= a.parent.row * a.parent.column) {
				return;
			}
			a.parent.$container.append(a._$widget);
			a.layoutIcons(a.normalWidth);

			this.$folderContainer.append(a.$wrapper);
			a._$widget.bind('click', function(event) {
				event.preventDefault();
				event.stopPropagation();
				a.open();
			});
			a.close();
			return this;
		};

		Folder.prototype.addSubWidget = function(widget) {
			this.subWidgets.push(widget);
		};

		Folder.prototype.reInit = function(widget) {
			this.widget = widget;
			this.blank = widget.title == '' && widget.icon == '';
			this.$title.html(widget.title);
			if (this.blank) {
				this.name = 'GridItem';
				this.$icon.empty();
				this.$icon.removeClass('folder_item_icon');
				this._$widget.unbind('click');
				this.subWidgets = [];
				this.widget.widgets = [];
			}
			this.isChanged = true;
			this._$widget.data('blank', this.blank);
			this.$widget = this._$widget;
		};

		Folder.prototype.exportWidget = function() {
			this.sortSubWidgets();
			this.widget.widgets = [];
			for (var i = 0; i < this.subWidgets.length; i++) {
				if (typeof this.subWidgets[i].exportWidget === 'function') {
					this.widget.widgets.push(this.subWidgets[i].exportWidget());
				}
			}
			return this.widget;
		};

		Folder.prototype.performClick = function() {
			this._$widget[0].click();
		};

		Folder.prototype.layoutIcons = function(width) {
			this.$icon.empty();
			for (var i = 0; this.subWidgets.length > 0
					&& i < this.subWidgets[0].subWidgets.length && i < 9; i++) {
				var gridItem = this.subWidgets[0].subWidgets[i];
				if (gridItem.blank) {
					continue;
				}
				var fw = width;
				var fc = 3;
				var fIconWH = 14;
				var margin = (fw - fc * fIconWH) / (fc + 1);
				var $fIcon = $('<div></div>').addClass('ficon').append($('<img />').attr('src', gridItem.widget.icon));
				$fIcon.css({
					top : margin + (fIconWH + margin) * parseInt(i / 3),
					left : margin + (fIconWH + margin) * (i % 3)
				});
				this.$icon.append($fIcon);
			}
		};

		Folder.prototype.open = function() {
			var a = this;
			a.$wrapper.show();
			this.$folderContainer.parent().css('background', 'url(/static/images/managecenter/background.png) no-repeat center');
			this.$folderContainer.parent().css('background-size', 'cover');
			this.$folderContainer.parent().show();
			this.resize();
			$(document).bind('click', function(event) {
				if ($(event.target).hasClass('panel')) {
					a.close();
				}
			});
		};

		Folder.prototype.close = function() {
			$(document).unbind('click');
			this.$wrapper.hide();
			try {
				this.$folderContainer.parent().hide();
			} catch (e) {
				
			}
			this.layoutIcons(this.normalWidth);
			if (this.widget.title != this.$hTitle.val()) {
				this.$title.html(this.$hTitle.val());
				this.widget.title = this.$hTitle.val();
				Widget.save();
			}
		};

		Folder.prototype.isEmpty = function() {
			var grid = this.subWidgets[0];
			for (var i = 0; i < grid.subWidgets.length; i++) {
				var gridItem = grid.subWidgets[i];
				if (!gridItem.blank) {
					return false;
				}
			}
			return true;
		};

		Folder.prototype.frame = function() {
			return {
				$widget : this._$widget,
				top : this.$wrapper.position().top,
				left : this.$wrapper.position().left,
				width : this.$wrapper.width(),
				height : this.$wrapper.height()
			};
		};

		Folder.prototype.resize = function() {
			this.$title.css({
				width : this.parent.colWidth * 0.75
			});
			var gHeight = this.subWidgets[0].rowHeight * 3 + 42;
			this.$widget.css({
				height : gHeight
			});
			var wrapperH = this.$header.height() + gHeight;
			var wrapperW = this.$wrapper.width();
			this.$wrapper.css({
				position : 'absolute',
				top : ($(window).height() - wrapperH) / 4,
				left : ($(window).width() - wrapperW) / 2,
				height : wrapperH
			});
			for (var i = 0; i < this.subWidgets.length; i++) {
				this.subWidgets[i].reLayout && this.subWidgets[i].reLayout(this.$widget.width());
			}
			for (var i = 0; i < this.subWidgets.length; i++) {
				this.subWidgets[i].resize();
			}
		};

		return Folder;

	})();

	Widget.register('Folder', Folder);

}).call(this);