(function() {
	'use strict';
	var $, Grid, GridItem;

	$ = jQuery;

	Grid = (function() {

		Grid.description = {
			name : 'Grid',
			desc : '表格',
			icon : ''
		};

		function Grid(widget, parent) {
			Widget.apply(this, arguments);

			this.settings = this.widget.settings || {};
			this.subWidgets = [];
			this.subWidgetsMap = {};
			this.row = this.settings.row || 3;
			this.column = this.settings.column || 3;
			this.colWidth = this.window().width / this.column;
			this.rowHeight = this.settings.rowH || 98;
			this.border = this.settings.border || false;
			this.draggingMoved = __bind(this.draggingMoved, this);
			this.draggingEnded = __bind(this.draggingEnded, this);
			this.draggingBegan = __bind(this.draggingBegan, this);
			this.position = __bind(this.position, this);
			this.layout = __bind(this.layout, this);

			this.$container = $('<div></div>').addClass('grid_container');
			this.$widget = $('<div></div>').attr('id', this.id).addClass('grid').append(this.$container);
			this.$dTraget = $('<div style="z-index:9999;"></div>').addClass('grid_item');

			this.style = widget.style || {};
			this.style.backgroundColor = this.style.backgroundColor || 'rgba(0, 0, 0, 0)';
			this.style.fontColor = this.style.fontColor || '#fdfdfd';
			this.style.borderColor = this.style.borderColor || '#787878';
			this.style.blankColor = this.style.blankColor || 'rgba(240, 240, 240, 0.2)';
			this.style.dropColor = this.style.dropColor || 'rgba(255, 255, 255, 1)';

			return this;
		}

		Grid.prototype.init = function() {
			this.parent.$widget.append(this.$widget);
			var $elements = this.elements();
			if (this.row == undefined) {
				if ($elements.length % this.column == 0) {
					this.row = $elements.length / this.column;
				} else {
					this.row = parseInt($elements.length / this.column) + 1;
				}
			}
			for (var i = $elements.length, j = this.row * this.column; i < j; i++) {
				var gridItem = new GridItem(GridItem.blank, this).init();
				this.addSubWidget(gridItem);
			}
			this.initDraggable();
			this.setStyle();
			return this;
		};

		Grid.prototype.addSubWidget = function(widget) {
			if (this.row && this.subWidgets.length >= this.row * this.column) {
				return;
			}
			this.subWidgets.push(widget);
			this.subWidgetsMap[widget.id] = widget;
		};

		Grid.prototype.exportWidget = function() {
			this.sortSubWidgets();
			this.widget.widgets = [];
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				if (typeof this.subWidgets[i].exportWidget === 'function') {
					this.widget.widgets.push(this.subWidgets[i].exportWidget());
				}
			}
			this.settings.row = this.row;
			this.settings.column = this.column;
			this.widget.settings = this.settings;
			this.widget.style = this.style;
			return this.widget;
		};

		Grid.prototype.show = function() {
			this.$widget.show();
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].show();
			}
		};

		Grid.prototype.hide = function() {
			this.$widget.hide();
		};

		Grid.prototype.elements = function() {
			return this.$container.children();
		};

		Grid.prototype.initDraggable = function(method) {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				new Draggable(this.subWidgets[i], {
					began : this.draggingBegan,
					ended : this.draggingEnded,
					moved : this.draggingMoved
				});
			}
			this.layout();
		};

		Grid.prototype.draggingBegan = function(event, $target) {
			this.appendDropContainer();
			var $dragging = $target;
			if ($dragging.data('blank') === 'true') {
				return;
			}
			this.folder = undefined;
			this.$dropContainer.data('position', $dragging.data('position'));
			this.sort(this.elements());
			this.$dTraget.html($target.html());
			this.$dTraget.hide();
			$('body').append(this.$dTraget);
		};

		Grid.prototype.draggingEnded = function(event, $target) {
			if (this.$dragging) {
				this.$dragging.data('position', this.$dropContainer.data('position'));
			}
			$('p', this.$dragging).show();
			this.$dragging = undefined;
			this.removeDropContainer();
			this.hideBlankContainer();
			this.moveTarget && this.moveTarget.move();
			this.$dTraget.remove();
			setTimeout(this.layout, 0);
		};

		Grid.prototype.draggingMoved = function(event, $target) {
			var $dragging = $target;
			if ($dragging.data('blank') === 'true') {
				return;
			}
			$('p', $dragging).hide();
			var dtp = this.getDTargetPosition(this);
			this.$dTraget.css({
				position : 'absolute',
				left : dtp.left + $dragging.position().left,
				top : dtp.top + $dragging.position().top,
				width : $dragging.width(),
				height : $dragging.height(),
				display : 'block'
			});
			this.$dropContainer.css({
				width : $dragging.width(),
				height : $dragging.height(),
				display : 'block'
			});
			$($('.centerr'), this.$dropContainer).css('border', '1px solid ' + this.style.dropColor);
			this.showBlankContainer();

			this.moveTarget && this.moveTarget.endMove();
			this.moveTarget = undefined;

			var x = parseInt($dragging.position().left / this.colWidth);
			x = x >= this.column ? this.column - 1 : x;
			var y = parseInt($dragging.position().top / this.rowHeight);
			y = y >= this.row ? this.row - 1 : y;

			var distance = parseInt(this.$dropContainer.data('position'));
			var $sortedEles = this.sort(this.elements());
			for (var i = 0, j = $sortedEles.length; i < j; i++) {
				var $element = $($sortedEles[i]);
				var xx = i % this.column;
				var yy = parseInt(i / this.column);
				if (x == xx && y == yy) {
					var dragingGI = this.subWidgetsMap[$dragging.attr('id')];
					var targetGI = this.subWidgetsMap[$element.attr('id')];
					var isInFolder = targetGI && targetGI.parent.parent.name === 'Folder';
					var r1 = 12;
					var r2 = 29;
					if (targetGI && (targetGI.blank || isInFolder) || dragingGI.name === 'Folder') {
						r1 = 0;
					}
					var left = $dragging.position().left - $element.position().left;
					var top = $dragging.position().top - $element.position().top;
					if (left > 0 && left < r1 && top > 0 && top < r1) {
						if (targetGI && targetGI.name === 'Folder' && dragingGI.name === 'GridItem') {
							this.moveTarget = dragingGI;
							this.moveTarget.moveType = GridItem.moveType.inFolder;
							this.moveTarget.folder = targetGI;
						}
						if (targetGI && targetGI.name === 'GridItem' && !targetGI.blank && !isInFolder && dragingGI.name === 'GridItem') {
							this.moveTarget = dragingGI;
							this.moveTarget.moveType = GridItem.moveType.inNewFolder;
							this.moveTarget.target = targetGI;
						}
					}
					if (left > r1 && left < r2 && top > r1 && top < r2) {
						$element.data('position', distance);
						distance = i;
					}
				}
			}
			if ((x + y * this.column) >= $sortedEles.length) {
				distance = $sortedEles.length - 1;
			}

			this.tryMoveInFolder($dragging);
			this.tryMoveOutFolder($dragging);
			this.tryMoveToOtherTab($dragging);
			this.tryMoveInNewFolder($dragging);

			distance = distance < 0 ? 0 : distance;
			distance = distance >= $sortedEles.length ? ($sortedEles.length - 1) : distance;
			this.$dropContainer.data('position', distance);
			if (this.border) {
				var borderStyle = '1px solid ' + this.style.borderColor;
				var position = distance;
				var col = this.column;
				var x = position % col;
				var y = parseInt(position / col);
				this.$dropContainer.css('border-top', y == 0 ? borderStyle : 'none');
				this.$dropContainer.css('border-right', x == col - 1 ? 'none' : borderStyle);
				this.$dropContainer.css('border-bottom', borderStyle);
			}

			this.$dragging = $dragging;
			this.layout();
		};

		Grid.prototype.position = function($elements) {
			var positions = [];
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				positions.push({
					x : this.colWidth * (i % this.column),
					y : this.rowHeight * parseInt(i / this.column),
					w : this.colWidth,
					h : this.rowHeight,
					$element : $element
				});
			}
			return positions;
		};

		Grid.prototype.layout = function() {
			var $elements = this.sort(this.elements());
			var positions = this.position($elements);
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				var position = positions[i];
				if ($element.is('.dragging')) {
					continue;
				}
				$element.css({
					position : 'absolute',
					left : position.x,
					top : position.y,
					width : position.w,
					height : position.h
				});
			}
			this.$container.css({
				height : this.row * this.rowHeight
			});
			this.$widget.css({
				height : this.row * this.rowHeight + 16
			});
			this.setStyle();
		};

		Grid.prototype.reLayout = function(width) {
			this.colWidth = (width || this.parent.$widget.width()) / this.column;
			this.layout();
		};

		Grid.prototype.findBlank = function() {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				var gridItem = this.subWidgets[i];
				if (gridItem.blank) {
					return gridItem;
				}
			}
			return null;
		};

		Grid.prototype.setStyle = function(key, value) {
			if (key && value) {
				this.style[key] = value;
			}
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].setStyle(key, value);
			}
		};

		Grid.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.column * this.colWidth,
				height : this.row * this.rowHeight
			};
		};

		Grid.prototype.resize = function() {
			this.reLayout();
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].resize();
			}
		};

		Grid.prototype.appendDropContainer = function() {
			var a = this;
			a.$dropContainer = $('<div></div>').addClass('drop_container_grid');
			a.$dropContainer.append($('<div></div>').addClass('centerr'));
			a.$dropContainer.css({
				width : 0,
				height : 0,
				position : 'absolute',
				display : 'none'
			});
			a.$container.append(a.$dropContainer);
		};

		Grid.prototype.removeDropContainer = function() {
			this.$dropContainer.remove();
		};

		Grid.prototype.showBlankContainer = function() {
			var $elements = this.elements().not('.dragging');
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				var gridItem = this.subWidgetsMap[$element.attr('id')];
				if (gridItem && gridItem.blank) {
					gridItem.$icon.addClass('blank_item_icon');
					gridItem.$icon.css('border', '1px solid ' + this.style.blankColor);
				}
			}
		};

		Grid.prototype.hideBlankContainer = function() {
			var $elements = this.elements();
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				var gridItem = this.subWidgetsMap[$element.attr('id')];
				if (gridItem && gridItem.blank) {
					gridItem.$icon.removeClass('blank_item_icon');
					gridItem.$icon.css('border', 'none');
				}
			}
		};

		Grid.prototype.getDTargetPosition = function(widget) {
			if (widget.name == 'Grid' && widget.$widget.css('position') === 'absolute') {
				return {
					top : widget.$widget.offset().top,
					left : widget.$widget.offset().left + widget.frame().left
				};
			} else if (widget.name == 'Folder' && widget.$wrapper.css('position') === 'absolute') {
				return {
					top : widget.$wrapper.offset().top + widget.$header.height() + 20,
					left : widget.$wrapper.offset().left
				};
			} else {
				return this.getDTargetPosition(widget.parent);
			}
		};

		Grid.prototype.tryMoveInFolder = function($dragging) {
			if (this.moveTarget
					&& this.moveTarget.moveType == GridItem.moveType.inFolder
					&& this.moveTarget.folder) {
				this.moveTarget.beginMove();
			}
		};

		Grid.prototype.tryMoveOutFolder = function($dragging) {
			var pWidget = this.parent;
			if (pWidget.name === 'Folder'
					&& ($dragging.position().top + $dragging.height() < 0 || $dragging.position().top + 20 > pWidget.$widget.height())) {
				this.moveTarget = this.subWidgetsMap[$dragging.attr('id')];
				this.moveTarget.moveType = GridItem.moveType.outFolder;
				this.moveTarget.beginMove();
			}
		};

		Grid.prototype.tryMoveToOtherTab = function($dragging) {
			var moveOrientation = -1;
			var moveTarget = this.subWidgetsMap[$dragging.attr('id')];
			var width = moveTarget != undefined ? moveTarget.$icon.width() : 0;
			var margin = ($dragging.width() - width) / 2;
			if ($dragging.position().left + width + margin > $(window).width()) {
				moveOrientation = GridItem.moveOrientation.right;
			}
			if ($dragging.position().left + margin < 0) {
				moveOrientation = GridItem.moveOrientation.left;
			}
			if (moveOrientation != -1) {
				this.moveTarget = moveTarget;
				this.moveTarget.moveType = GridItem.moveType.toOtherTab;
				this.moveTarget.moveOrientation = moveOrientation;
				this.moveTarget.beginMove();
			}
		};

		Grid.prototype.tryMoveInNewFolder = function($dragging) {
			if (this.moveTarget
					&& this.moveTarget.moveType == GridItem.moveType.inNewFolder
					&& this.moveTarget.target) {
				this.moveTarget.beginMove();
			}
		};

		return Grid;

	})();

	Widget.register('Grid', Grid);

	GridItem = (function() {

		GridItem.description = {
			name : 'GridItem',
			desc : '单元格',
			icon : ''
		};

		GridItem.blank = {
			name : 'GridItem',
			title : '',
			icon : '',
			iconSelected : '',
			url : ''
		};

		GridItem.moveType = {
			inFolder : 0,
			outFolder : 1,
			toOtherTab : 2,
			inNewFolder : 3
		};

		GridItem.moveOrientation = {
			left : 0,
			right : 1
		};

		GridItem.userType = {
			tourist : '01',
			login : '02',
			enterprise : '03',
			individual : '04'
		};

		function GridItem(widget, parent) {
			Widget.apply(this, arguments);

			this.imgLoaded = false;
			this.login = widget.login == 'Y';
			this.border = parent.border || false;
			this.userType = widget.userType || '01';
			this.blank = widget.title == '' && widget.icon == '';
			this.serverId = this.widget.serverId || Portal.app.serverId;
			this.serverType = '1';
			this.serverUrl = this.widget.serverUrl || Portal.server.gsServerUrl;
			// this.serverUrl = this.serverType == '1' ? Portal.server.gsServerUrl : Portal.server.dsServerUrl;
			Widget.servers[this.serverId + '-' + this.serverType] = {
				serverId : this.serverId,
				serverType : this.serverType,
				serverUrl : this.serverUrl
			};

			this.$icon = $('<div></div>').addClass('grid_item_icon');
			this.$badge = $('<span>12</span>').addClass('badge');
			this.$title = $('<p></p>').css({
				width : this.parent.colWidth * 0.75
			}).html(widget.title);
			this.$widget = $('<div></div>').attr('id', this.id).data('blank',
					this.blank).addClass('grid_item').append(this.$icon).append(this.$title);

			this.style = parent.style || {};

			return this;
		}

		GridItem.prototype.init = function() {
			var a = this;
			if (a.parent.row && a.parent.elements().length >= a.parent.row * a.parent.column) {
				return;
			}
			a.parent.$container.append(a.$widget);
			a.$widget.bind('click', function(event) {
				event.preventDefault();
				event.stopPropagation();
				a.open();
			});
			// this.setStyle();
			return this;
		};

		GridItem.prototype.reInit = function(widget) {
			this.widget = widget;
			this.blank = widget.title == '' && widget.icon == '';
			this.$title.html(widget.title);
			if (!this.blank && widget.icon) {
				this.$icon.append($('<img />').attr('src', widget.icon));
			} else {
				this.$icon.empty();
				this.$widget.unbind('click');
			}
			this.isChanged = true;
			this.$widget.data('blank', this.blank);
		};

		GridItem.prototype.exportWidget = function() {
			return this.widget;
		};

		GridItem.prototype.show = function() {
			if (!this.blank && this.widget.icon && !this.imgLoaded) {
				this.$icon.append($('<img />').attr('src', this.widget.icon));
				this.showBadge();
				this.imgLoaded = true;
			}
			if (this.subWidgets && this.subWidgets.length > 0) {
				for (var i = 0, j = this.subWidgets.length; i < j; i++) {
					this.subWidgets[i].show();
				}
			}
		};
		
		GridItem.prototype.showBadge = function() {
			var a = this;
			var url = this.widget.badgeUrl;
			if (!url) {
				return;
			}
			var callback = function(d) {
				if (d.number && d.number > 0) {
					a.$badge.html(d.number > 99 ? 99 : d.number);
					a.$icon.append(a.$badge);
				}
			};
			FXUI.post(url, {}, callback);
		};

		GridItem.prototype.open = function() {
			Portal.goPage({
				uuid : this.widget.uuid,
				url : this.widget.url,
				serverId : this.serverId,
				serverUrl : this.serverUrl,
				serverType : this.serverType,
				userType : this.userType,
				redirectTips : this.widget.redirectTips,
                blankTips : this.widget.blankTips,
				confirmText : this.widget.confirmText,
                title : this.widget.title
			});
		};

		GridItem.prototype.performClick = function() {
			// this.$widget[0].click();
			this.open();
		};

		GridItem.prototype.setStyle = function(key, value) {
			if (key != undefined && value != undefined) {
				this.style[key] = value;
			}
			this.frame().$widget.css('background', this.style.backgroundColor);
			this.$title.css('color', this.style.fontColor);
			if (this.border) {
				var borderStyle = '1px solid ' + this.style.borderColor;
				var position = this.frame().$widget.data('position');
				var col = this.parent.column;
				var x = position % col;
				var y = parseInt(position / col);
				this.frame().$widget.css('border-top', y == 0 ? borderStyle : 'none');
				this.frame().$widget.css('border-right', x == col - 1 ? 'none' : borderStyle);
				this.frame().$widget.css('border-bottom', borderStyle);
				if (this.frame().$widget.is('.dragging')) {
					this.frame().$widget.css('border', 'none');
				}
			}
		};

		GridItem.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.$widget.width(),
				height : this.$widget.height()
			};
		};

		GridItem.prototype.resize = function() {
			this.$title.css({
				width : this.parent.colWidth * 0.75
			});
		};

		GridItem.prototype.beginMove = function() {
			switch (this.moveType) {
			case GridItem.moveType.inFolder:
				this.beginMoveInFolder();
				break;
			case GridItem.moveType.outFolder:
				this.beginMoveOutFolder();
				break;
			case GridItem.moveType.toOtherTab:
				this.beginMoveToOtherTab();
				break;
			case GridItem.moveType.inNewFolder:
				this.beginMoveInNewFolder();
				break;
			}
		};

		GridItem.prototype.endMove = function() {
			switch (this.moveType) {
			case GridItem.moveType.inFolder:
				this.endMoveInFolder();
				break;
			case GridItem.moveType.outFolder:
				this.endMoveOutFolder();
				break;
			case GridItem.moveType.toOtherTab:
				this.endMoveToOtherTab();
				break;
			case GridItem.moveType.inNewFolder:
				this.endMoveInFolder();
				break;
			}
		};

		GridItem.prototype.move = function() {
			switch (this.moveType) {
			case GridItem.moveType.inFolder:
				this.moveInFolder();
				break;
			case GridItem.moveType.outFolder:
				this.moveOutFolder();
				break;
			case GridItem.moveType.toOtherTab:
				this.moveToOtherTab();
				break;
			case GridItem.moveType.inNewFolder:
				this.moveInFolder();
				break;
			}
		};

		GridItem.prototype.beginMoveInFolder = function() {
			this.folder.$icon.addClass('folder_item_icon_drop');
			this.folder.layoutIcons(this.folder.dropWidth);
		};

		GridItem.prototype.endMoveInFolder = function() {
			this.folder.$icon.removeClass('folder_item_icon_drop');
			this.folder.layoutIcons(this.folder.normalWidth);
		};

		GridItem.prototype.moveInFolder = function() {
			var isDropped = false;
			for (var i = 0, j = this.folder.subWidgets.length; i < j; i++) {
				var grid = this.folder.subWidgets[i];
				var gridItem = grid.findBlank();
				if (gridItem != null) {
					gridItem.reInit(this.widget);
					isDropped = true;
					break;
				}
			}
			if (!isDropped && this.folder.$pager) {
				var gridWidget = {
					name : 'Grid',
					title : '表格',
					settings : {
						row : 3,
						column : 3
					},
					widgets : [ this.widget ]
				};
				var grid = Widget.resolve(gridWidget, this.folder);
				this.folder.addSubWidget(grid);
				isDropped = true;
			}
			if (isDropped) {
				this.reInit(GridItem.blank);
			}
			this.endMoveInFolder();
		};

		GridItem.prototype.beginMoveOutFolder = function() {
			var folder = this.parent.parent;
			folder.$widget.addClass('folder_body_move_out');
			Message.show(Message.m2);
		};

		GridItem.prototype.endMoveOutFolder = function() {
			var folder = this.parent.parent;
			folder.$widget.removeClass('folder_body_move_out');
			if (folder.isEmpty()) {
				folder.close();
				folder.reInit(GridItem.blank);
			}
			Message.hide();
		};

		GridItem.prototype.moveOutFolder = function() {
			var folder = this.parent.parent;
			var toGrid = folder.parent;
			for (var i = 0, j = toGrid.subWidgets.length; i < j; i++) {
				var gridItem = toGrid.subWidgets[i];
				if (gridItem.blank) {
					gridItem.reInit(this.widget);
					this.reInit(GridItem.blank);
					folder.layoutIcons(folder.normalWidth);
					break;
				}
			}
			this.endMoveOutFolder();
		};

		GridItem.prototype.beginMoveToOtherTab = function() {
			if (this.moveOrientation == undefined || this.moveOrientation == -1) {
				return;
			}
			var pWidget = this;
			while (pWidget && pWidget.name != 'TabBarItem') {
				pWidget = pWidget.parent;
			}
			if (pWidget && pWidget.name == 'TabBarItem') {
				this.tabBarItem = pWidget;
			}
			if (!this.tabBarItem) {
				return;
			}
			if ((this.moveOrientation == GridItem.moveOrientation.left && this.tabBarItem
					.data4position() > 0)
					|| (this.moveOrientation == GridItem.moveOrientation.right && this.tabBarItem
							.data4position() < this.tabBarItem.parent.subWidgets.length - 1)) {
				Message.hide().show(Message.m3);
			}
		};

		GridItem.prototype.endMoveToOtherTab = function() {

		};

		GridItem.prototype.moveToOtherTab = function() {
			if (!this.tabBarItem) {
				return;
			}
			var toTabPosition = -1;
			if (this.moveOrientation == GridItem.moveOrientation.left && this.tabBarItem.data4position() > 0) {
				var toTabPosition = this.tabBarItem.data4position() - 1;
			} else if (this.moveOrientation == GridItem.moveOrientation.right
					&& this.tabBarItem.data4position() < this.tabBarItem.parent.subWidgets.length - 1) {
				var toTabPosition = this.tabBarItem.data4position() + 1;
			}
			if (toTabPosition == -1) {
				return;
			}
			var toTabBarItem = this.tabBarItem.parent.findTabBarItem(toTabPosition);
			var toTabContent = toTabBarItem.subWidget;

			var findTargetGrid = function(widget) {
				if (!widget.subWidgets) {
					return null;
				}
				for (var i = 0, j = widget.subWidgets.length; i < j; i++) {
					var sWidget = widget.subWidgets[i];
					if (sWidget.name == 'Grid') {
						var gridItem = sWidget.findBlank();
						if (gridItem != null) {
							return sWidget;
						}
					} else {
						var grid = findTargetGrid(sWidget);
						if (grid != null) {
							return grid;
						}
					}
				}
				return null;
			};
			var targetGrid = findTargetGrid(toTabContent);
			if (targetGrid == null) {
				Message.hide().show(Message.m4);
				return;
			}
			var gridItem = targetGrid.findBlank();
			if (this.name == 'GridItem') {
				gridItem.reInit(this.widget);
			} else if (this.name == 'Folder') {
				var position = parseInt(gridItem.$widget.data('position'));
				targetGrid.subWidgets.splice(position, 1);
				delete targetGrid.subWidgetsMap[gridItem.id];

				var folder = Widget.resolve(this.widget, targetGrid);
				folder._$widget.data('position', position);

				targetGrid.subWidgets.splice(position, 0, folder);
				targetGrid.subWidgetsMap[folder.id] = folder;

				gridItem.$widget.remove();

				new Draggable(folder, {
					began : targetGrid.draggingBegan,
					ended : targetGrid.draggingEnded,
					moved : targetGrid.draggingMoved
				});
				targetGrid.layout();
			}
			this.reInit(GridItem.blank);
			this.endMoveToOtherTab();
		};

		GridItem.prototype.beginMoveInNewFolder = function() {
			var targetGrid = this.parent;
			var position = parseInt(this.target.$widget.data('position'));
			targetGrid.subWidgets.splice(position, 1);
			delete targetGrid.subWidgetsMap[this.target.id];

			var folderWidget = {
				name : 'Folder',
				title : '文件夹',
				widgets : [ {
					name : 'Grid',
					title : '表格',
					settings : {
						row : 3,
						column : 3
					},
					widgets : [ this.target.widget ]
				} ]
			};
			this.folder = Widget.resolve(folderWidget, targetGrid);
			this.folder._$widget.data('position', position);

			targetGrid.subWidgets.splice(position, 0, this.folder);
			targetGrid.subWidgetsMap[this.folder.id] = this.folder;

			this.target.$widget.remove();

			new Draggable(this.folder, {
				began : targetGrid.draggingBegan,
				ended : targetGrid.draggingEnded,
				moved : targetGrid.draggingMoved
			});
			targetGrid.layout();

			this.folder.$icon.addClass('folder_item_icon_drop');
		};

		return GridItem;

	})();

	Widget.register('GridItem', GridItem);

}).call(this);