(function() {
	'use strict';
	var $ = jQuery, homeMenuItemMap = {}, menuItemMap = {};

	function HomeMenuEdit() {
		this.widget = Widget.widget.widget;
		this.homeMenu = undefined;
		this.homeMenuItems = [];
		this.usedAlot = undefined;
		this.menus = [];
		this.edited = false;

		this.back = __bind(this.back, this);
		this.draggingBegan = __bind(this.draggingBegan, this);
		this.draggingEnded = __bind(this.draggingEnded, this);
		this.draggingMoved = __bind(this.draggingMoved, this);

		var $page = $('#menuedit');
		this.$homeMenuGrid = $('.home-menu-grid>ul', $page);
		this.$content = $('.content', $page);

		$('.pull-left', $page).click(this.back);
	}

	HomeMenuEdit.prototype.init = function() {
		$.showIndicator();
		this.initHomeMenu();
		this.initMenu();
		$.hideIndicator();
	};

	HomeMenuEdit.prototype.initHomeMenu = function() {
		this.homeMenu = this.getHomeMenu();
		this.homeMenuItems = [];
		this.$homeMenuGrid.empty();
		for (var i = 0, j = this.homeMenu.widgets.length; i < j; i++) {
			var widget = this.homeMenu.widgets[i];
			if (widget.name === 'UsedAlot') {
				this.usedAlot = widget;
				continue;
			}
			if (widget.icon === '' && widget.title === '') {
				continue;
			}
			var menuItem = new HomeMenuItem(widget, this).init();
			this.homeMenuItems.push(menuItem);
		}
		this.sortMenuItems();
	};

	HomeMenuEdit.prototype.initMenu = function() {
		var menus = [];
		var resolve = function(widget, parent) {
			if (widget.name === 'GridV2' || widget.name === 'Menu') {
				var menu = {
					title : parent && parent.title,
					widget : widget
				};
				menus.push(menu);
			}
			if (widget.widgets && widget.widgets.length > 0) {
				for (var i = 0, j = widget.widgets.length; i < j; i++) {
					resolve(widget.widgets[i], widget);
				}
			}
		};
		resolve(Portal.template.widget);
		this.$content.empty();
		for (var i = 0, j = menus.length; i < j; i++) {
			this.menus.push(new Menu(menus[i], this).init());
		}
	};

	HomeMenuEdit.prototype.back = function() {
		this.turnOffDrag();
		this.edited && this.reLayoutHomeGrid();
		$.router.back();
	};

	HomeMenuEdit.prototype.reLayoutHomeGrid = function() {
		var resolve = function(widget) {
			if (widget.name === 'GridV2') {
				return widget;
			}
			var widgets = [];
			widget.subWidget && widgets.push(widget.subWidget);
			widget.subWidgets && widget.subWidgets.length > 0
					&& (widgets = widget.subWidgets.concat());
			for (var i = 0, j = widgets.length; i < j; i++) {
				var ww = resolve(widgets[i]);
				if (ww) {
					return ww;
				}
			}
			return null;
		};
		var grid = resolve(Widget.widget);
		grid && grid.reLoad(this.homeMenu);
	};
	
	HomeMenuEdit.prototype.getHomeMenu = function() {
		var resolve = function(widget) {
			if (widget.name === 'GridV2') {
				return widget;
			}
			if (widget.widgets && widget.widgets.length > 0) {
				for (var i = 0, j = widget.widgets.length; i < j; i++) {
					var ww = resolve(widget.widgets[i]);
					if (ww) {
						return ww;
					}
				}
			}
			return null;
		};
		return resolve(this.widget);
	};

	HomeMenuEdit.prototype.turnOnDrag = function() {
		for (var i = 0, j = this.homeMenuItems.length; i < j; i++) {
			this.homeMenuItems[i].offEvents();
			this.homeMenuItems[i].shake();
		}
		this.isDragOn = true;
		this.initDraggable();
	};

	HomeMenuEdit.prototype.turnOffDrag = function() {
		for (var i = 0, j = this.homeMenuItems.length; i < j; i++) {
			this.homeMenuItems[i].initDragSwith();
			this.homeMenuItems[i].unshake();
		}
		this.isDragOn = false;
	};

	HomeMenuEdit.prototype.initDraggable = function() {
		for (var i = 0, j = this.homeMenuItems.length; i < j; i++) {
			var menuItem = this.homeMenuItems[i];
			menuItem.widget.draggable && new Draggable2(menuItem.$widget, {
				began : this.draggingBegan,
				ended : this.draggingEnded,
				moved : this.draggingMoved
			});
		}
	};
	
	HomeMenuEdit.prototype.draggingBegan = function(event, $target) {
		var wHeight = 0;
		this.widgetPosition = [];
		for (var i in this.homeMenuItems) {
			var $widget = this.homeMenuItems[i].$widget;
			var top = $widget.offset().top;
			var left = $widget.offset().left;
			var height = $widget.height();
			var width = $widget.width();
			this.widgetPosition.push([top, top + height, left, left + width]);
			wHeight = height;
		}
		this.$dropTarget = $('<li class="home-menu-grid-item drop"><div class="wrapper"><div class="icon"></div><p>&nbsp;</p></div></li>');
	};

	HomeMenuEdit.prototype.draggingEnded = function(event, $target) {
		if (this.$dropTarget && this.dropIndex) {
			$target.removeClass('hide').detach();
			this.$dropTarget.before($target);
			this.$dropTarget.detach();
			this.spliceMenuItem(parseInt($target.attr('index')), this.dropIndex);
			this.sortMenuItems();
			delete this.$dropTarget;
			delete this.dropIndex;
			this.saveHomeMenu();
			this.reLayoutHomeGrid();
		}
	};

	HomeMenuEdit.prototype.draggingMoved = function(event, $target, $dragging) {
		$dragging.width($(window).width() / 3);
		var index = -1;
		var targetTop = $dragging.position().top;
		var targetLeft = $dragging.position().left;
		for (var i in this.widgetPosition) {
			var p = this.widgetPosition[i];
			if (targetTop >= p[0] && targetTop <= p[1] && targetLeft >= p[2] && targetLeft <= p[3]) {
				index = i;
			}
		}
		if (index == -1 || !this.homeMenuItems[index].widget.draggable) {
			return;
		}
		var targetIndex = parseInt($target.attr('index'));
		$target.addClass('hide');
		this.$dropTarget.detach();
		var $items = this.$homeMenuGrid.children();
		if (index < targetIndex) {
			$($items[index]).before(this.$dropTarget);
		} else {
			$($items[index]).after(this.$dropTarget);
		}
		this.dropIndex = index;
	};

	HomeMenuEdit.prototype.sortMenuItems = function() {
		homeMenuItemMap = {};
		for (var i in this.homeMenuItems) {
			var hmi = this.homeMenuItems[i];
			hmi.$widget.attr('index', i);
			homeMenuItemMap[hmi.id] = hmi;
		}
	};
	
	HomeMenuEdit.prototype.spliceMenuItem = function(sourceIndex, targetIndex) {
		var widget = this.homeMenuItems.splice(sourceIndex, 1);
		this.homeMenuItems.splice(targetIndex, 0, widget[0]);
	};

	HomeMenuEdit.prototype.removeHomeMenuItem =  function(index) {
		this.homeMenuItems.splice(index, 1);
		this.sortMenuItems();
		this.saveHomeMenu();
		this.reLayoutHomeGrid();
	}

	HomeMenuEdit.prototype.addHomeMenuItem =  function(widget) {
		var menuItem = new HomeMenuItem(widget, this);
		var $items = this.$homeMenuGrid.children();
		$($items[this.homeMenuItems.length - 1]).after(menuItem.$widget);
		$('.drop', this.$homeMenuGrid).last().remove();
		this.homeMenuItems.push(menuItem);
		this.sortMenuItems();
		this.saveHomeMenu();
		this.reLayoutHomeGrid();
	}

	HomeMenuEdit.prototype.saveHomeMenu =  function() {
		var widgets = [];
		for (var i = 0, j = this.homeMenuItems.length; i < j; i++) {
			widgets.push(this.homeMenuItems[i].widget);
		}
		this.usedAlot && widgets.push(this.usedAlot);
		this.homeMenu.widgets = widgets;
		var template = {
			id : Widget.templateId,
			version : Widget.version,
			widget : this.widget
		};
		Widget.storage[Widget.templateId] = JSON.stringify(template);
		this.edited = true;
	}

	function HomeMenuItem(widget, parent) {
		this.id = widget.uuid;
		this.widget = widget;
		this.parent = parent;
		this.$icon = $('<div class="icon"></div>').append($('<img />').attr('src', widget.icon));
		this.$title = $('<p></p>').html(widget.title);
		this.$del = $('<a class="edit icon2-del"></a>');
		this.$wrapper = $('<div class="wrapper"></div>').append(this.$icon).append(this.$title);
		this.$widget = $('<li class="home-menu-grid-item"></li>').append(this.$wrapper);

		if (widget.draggable) {
			var self = this;
			this.$wrapper.append(this.$del);
			this.$del.click(function() {
				self.del();
			});
		}

		this.initDragSwith();
		return this;
	}

	HomeMenuItem.prototype.init = function() {
		this.parent.$homeMenuGrid.append(this.$widget);
		return this;
	}

	HomeMenuItem.prototype.coordinate = function(event) {
		switch (event.type) {
		case 'touchstart':
		case 'touchmove':
		case 'touchend':
		case 'touchcancel':
			if (event instanceof TouchEvent) {
				return event.touches[0];
			}
			return event.originalEvent.touches[0];
		default:
			return event;
		}
	};

	HomeMenuItem.prototype.initDragSwith = function() {
		this.offEvents();
		var self = this;
		var c;
		var reset = function() {
			c && clearTimeout(c);
			c = 0;
		};
		this.$widget.on('mousedown touchstart', function() {
			self.touchPisition = {
				x : self.coordinate(event).pageX,
				y : self.coordinate(event).pageY
			};
			c = setTimeout(function() {
				reset();
				self.parent.turnOnDrag();
			}, 1200);
		});
		this.$widget.on('mousemove touchmove', function(event) {
			self.isMove(event) && reset();
		});
		this.$widget.on('mouseup touchend touchcancel', function(event) {
			delete self.touchPisition;
			self.isMove(event) && reset();
		});
	};

	HomeMenuItem.prototype.isMove = function(event) {
		if (!this.touchPisition) {
			return;
		}
		return Math.abs(this.coordinate(event).pageX - this.touchPisition.x) > 4 || Math.abs(this.coordinate(event).pageY - this.touchPisition.y) > 4;
	};

	HomeMenuItem.prototype.offEvents = function() {
		this.$widget.off('mousedown touchstart mousemove touchmove mouseup touchend touchcancel');
	};

	HomeMenuItem.prototype.shake = function() {
		if (!this.widget.draggable) {
			return;
		}
		var self = this;
		var $img = $('img', this.$icon);
		var r1 = function() {
			$img.removeClass('r-right').addClass('r-left');
			self._shake = setTimeout(function() {
				r2();
			}, 100);
		};
		var r2 = function() {
			$img.removeClass('r-left').addClass('r-right');
			self._shake = setTimeout(function() {
				r1();
			}, 100);
		};
		r1();
	};

	HomeMenuItem.prototype.unshake = function() {
		$('img', this.$icon).removeClass('r-left').removeClass('r-right');
		this._shake && clearTimeout(this._shake);
	};

	HomeMenuItem.prototype.del = function() {
		var index = this.$widget.attr('index');
		this.$widget.remove();
		var $items = this.parent.$homeMenuGrid.children();
		$($items[$items.length - 1]).after($('<li class="home-menu-grid-item drop"><div class="wrapper"><div class="icon"></div><p>&nbsp;</p></div></li>'));
		this.parent.removeHomeMenuItem(index);
		menuItemMap[this.id].refresh();
	};

	function Menu(menu, parent) {
		this.menu = menu;
		this.parent = parent;
		this.menus = [];
		this.$widget = $('<div class="menu-edit"></div>');
		this.$title = $('<div class="menu-edit-title"></div>').append($('<h1></h1>').html(menu.title));
		this.$items = $('<div class="menu-edit-items"></div>');
		this.$itemsUl = $('<ul></ul>').appendTo(this.$items);
		this.$widget.append(this.$title).append(this.$items);

		return this;
	}

	Menu.prototype.init = function() {
		this.parent.$content.append(this.$widget);
		this.initMenus();
		return this;
	}
	
	Menu.prototype.initMenus = function() {
		var menus = [];
		var widget = this.menu.widget;
		if (widget.name === 'GridV2') {
			menus = widget.widgets;
		} else if (widget.name === 'Menu') {
			var resolve = function(widget) {
				if (widget.name === 'MenuItem') {
					var copy = $.extend(true, {}, widget);
					copy.name = 'GridItemV2';
					copy.draggable = true;
					menus.push(copy);
				}
				if (widget.widgets && widget.widgets.length > 0) {
					for (var i = 0, j = widget.widgets.length; i < j; i++) {
						resolve(widget.widgets[i]);
					}
				}
			};
			resolve(widget);
		}
		for (var i = 0, j = menus.length; i < j; i++) {
			if (menus[i].name === 'UsedAlot') {
				continue;
			}
			if (menus[i].icon === '' && menus[i].title === '') {
				continue;
			}
			var menu = new MenuItem(menus[i], this).init();
			this.menus.push(menu);
			menuItemMap[menu.id] = menu;
		}
	};

	function MenuItem(widget, parent) {
		this.id = widget.uuid;
		this.widget = widget;
		this.parent = parent;
		this.$icon = $('<div class="icon"></div>').append($('<img />').attr('src', widget.icon));
		this.$title = $('<p></p>').html(widget.title);
		this.$del = $('<a class="edit icon2-del"></a>');
		this.$add = $('<a class="edit icon2-add"></a>');
		this.$wrapper = $('<div class="wrapper"></div>').append(this.$icon).append(this.$title);
		this.$widget = $('<li class="menu-edit-item"></li>').append(this.$wrapper);

		var self = this;
		this.$del.click(function() {
			self.del();
		});
		this.$add.click(function() {
			self.add();
		});

		return this;
	}

	MenuItem.prototype.init = function() {
		this.parent.$itemsUl.append(this.$widget);
		this.refresh();
		return this;
	};

	MenuItem.prototype.refresh = function() {
		this.$del.detach();
		this.$add.detach();
		if (homeMenuItemMap[this.id]) {
			this.widget.draggable && this.$wrapper.append(this.$del);
		} else {
			this.$wrapper.append(this.$add);
		}
	};

	MenuItem.prototype.del = function() {
		homeMenuItemMap[this.id].del();
	};

	MenuItem.prototype.add = function() {
		var hme = window.HomeMenuEdit;
		var maxLength = hme.homeMenu.settings.row * hme.homeMenu.settings.column - 1;
		if (hme.homeMenuItems.length >= maxLength) {
			$.toast('首页最多添加' + maxLength + '个常用菜单');
			return;
		}
		hme.addHomeMenuItem(this.widget);
		this.refresh();
	};

	window.HomeMenuEdit = new HomeMenuEdit();
}).call(this);