(function() {
	'use strict';
	var $, HomeNav, storage;

	$ = jQuery;
	storage = window.localStorage;

	HomeNav = (function() {

		HomeNav.description = {
			name : 'HomeNav',
			desc : '首页导航条'
		};

		function HomeNav(widget, parent) {
			Widget.apply(this, arguments);

			this.titleSize = 0.8;

			this.openMore = __bind(this.openMore, this);
			this.closeMore = __bind(this.closeMore, this);

			this.$widget = $('<div class="homenav"></div>');
			this.$header = $('#header').removeClass('mp-header');
			this.$logo = $('<div class="logo"><img src="/static/images/common/logo.png"/></div>');
			this.$title = $('<div class="home-title"></div>').html(this.resolveTitle(this.widget.title));
			this.$login = $('<span class="icon2-login"></span>');
			this.$logout = $('<a class="logout">退出</a>');
			this.$more = $('<span class="icon2-more"></span>');
			this.$moreView = $('<div class="more-view"><span class="icon2-triangle"></span><div>');
			this.$moreMenu = $('<div class="more-menu"></div>').appendTo(this.$moreView);
			this.$overlay = $('<div class="panel-overlay"></div>');

			this.style = widget.style || {};
			this.style.titleSize = this.style.titleSize || 1;

			return this;
		}

		HomeNav.prototype.init = function() {
			return this;
		};

		HomeNav.prototype.exportWidget = function() {
			this.widget.style = this.style;
			return this.widget;
		};

		HomeNav.prototype.show = function() {
			this.$header.empty().append(this.$logo).append(this.$title).append(this.$more);
			this.setHeader();
			this.setStyle();
			this.setLogin();
			var self = this;
			Portal.loadCallback(function() {
				self.setLogin();
			});
			this.createMoreMenu();
		};

		HomeNav.prototype.hide = function() {
			
		};

		HomeNav.prototype.setHeader = function() {
			if (this.widget.headerHidden === true) {
				this.$header.hide();
				$('#content').css('top', 0);
			} else {
				this.$header.show();
				$('#content').css('top', '2.2rem');
			}
		};

		HomeNav.prototype.resolveTitle = function(title) {
			if (title && title.indexOf('#') > 0) {
				var titles = title.split('#');
				title = '<p>' + titles[0] + '</p>' + '<p>' + titles[1] + '</p>';
				this.titleSize = 0.75;
			}
			return title;
		};

		HomeNav.prototype.setStyle = function(key, value) {
			if (key && value) {
				this.style[key] = value;
			}
			this.$title.css({'fontSize' : (this.titleSize * this.style.titleSize) + 'rem'});
		};

		HomeNav.prototype.setLogin = function() {
			if (FXUI.isWx()) {
				return;
			}
			if (Portal.userDetails) {
				this.$login.detach();
				this.$more.before(this.$logout);
				this.$logout.on('click', function() {
					Portal.logout();
				});
			} else {
				this.$logout.detach();
				this.$more.before(this.$login);
				this.$login.on('click', function() {
					FXUI.goPage('/app/assets/user/login.html?FxAppId=' + Portal.appId + '&page=' + Portal.page);
				});
			}
		};

		HomeNav.prototype.createMoreMenu = function() {
			var self = this;
			this.$more.click(this.openMore);
			var $ul = $('<ul class="more-menu-ul"></ul>');
			$('<li><span class="icon2-search"></span><a>搜索</a></li>').appendTo($ul).click(function(event) {
				self.closeMore();
				self.openSearchPage();
			});
			$('<li><span class="icon2-scan"></span><a>扫一扫</a></li>').appendTo($ul).click(function(event) {
				self.closeMore();
				self.scan();
			});
			this.$moreMenu.empty().append($ul);
		};

		HomeNav.prototype.openMore = function() {
			$('body').append(this.$overlay).append(this.$moreView);
			this.$overlay.on('click', this.closeMore);
		};

		HomeNav.prototype.closeMore = function() {
			this.$moreView.detach();
			this.$overlay.off('click').detach();
		};

		HomeNav.prototype.openSearchPage = function() {
			$.showIndicator();
			loadScript('/static/js/assets/portal/search.js', function() {
				$.hideIndicator();
				$.router.loadPage('#search');
				setTimeout(function() {
					Search.init();
				}, 300);
			});
		};

		HomeNav.prototype.scan = function() {
			var scanComplete = function(text) {
				if (text.indexOf('http') === 0 || text.indexOf('/') === 0) {
					window.location.href = text;
				} else {
					$.alert(text, '扫描结果');
				}
			};
			if (FXUI.isWx()) {
				var self = this;
				$.showIndicator();
				loadScript(window.location.protocol + '//res.wx.qq.com/open/js/jweixin-1.4.0.js', function() {
					var wxScan = function() {
						$.hideIndicator();
						wx.scanQRCode({
							needResult : 1,
							scanType : [ 'qrCode', 'barCode' ],
							success : function(res) {
								scanComplete(res.resultStr);
							}
						});
					};
					if (!self.isWxConfig) {
						FXUI.wxConfig(function() {
							self.isWxConfig = true;
							wxScan();
						});
					} else {
						wxScan();
					}
				});
			} else {
				FXAPP.scanQRCode({}, function(data) {
					scanComplete(data.text);
				});
			}
		};

		HomeNav.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.height
			};
		};

		return HomeNav;

	})();

	Widget.register('HomeNav', HomeNav);
}).call(this);