(function() {
	'use strict';
	var $, List, ListItem, Converter = {};

	$ = jQuery;

	List = (function() {

		function List(widget, parent) {
			Widget.apply(this, arguments);

			this.title = widget.title || '列表标题';
			this.width = this.window().width;
			this.height = 160;
			this.url = widget.url;
			this.limit = widget.limit || 4;
			this.converter = widget.converter || '';
			this.page = widget.page;

			this.$widget = $('<div></div>').attr('id', this.id).addClass('listz');
			this.$title = $('<div></div>').addClass('listz-title');
			this.$title.append('<span class="icon2-title"></span>').append(this.title).appendTo(this.$widget);
			this.$listBlock = $('<div></div>').addClass('list-block').appendTo(this.$widget);
			this.$ul = $('<ul></ul>').appendTo(this.$listBlock);
			
			this.style = widget.style || {};
			this.style.width = this.width;
			this.style.height = this.style.height || this.height;
			
			return this;
		}

		List.prototype.init = function() {
			this.parent.$widget.append(this.$widget);
			this.setStyle();
			this.getData();
			return this;
		};

		List.prototype.exportWidget = function() {
			this.widget.widgets = [];
			this.widget.style = this.style;
			return this.widget;
		};

		List.prototype.getData = function() {
			if (!this.url) {
				return;
			}
			var that = this;
			var url = Portal.contextPath + '/index/list.do';
			var params = {
				url : this.url
			};
			var data = {
				params : encodeURIComponent(encodeURIComponent(JSON.stringify(params)))
			};
			var callback = function(d) {
				that.$ul.empty().removeClass('loading');
				if (d.flag == 'ok') {
					that.appendData(d.data);
				} else {
					that.$ul.addClass('error').append($('<li></li>').html(d.errMsg));
				}
			};
			this.$ul.removeClass('error').addClass('loading').empty().append($('<li>&nbsp;</li>'));
			FXUI.post(url, data, callback);
		};
		
		List.prototype.appendData = function(data) {
			if (this.converter) {
				data = new Converter[this.converter](data).convert();
			}
			if (Object.prototype.toString.call(data) == '[object Array]') {
				for (var i in data) {
					if (i >= this.limit) {
						break;
					}
					var listItem = new ListItem(data[i], this);
					this.$ul.append(listItem.$li);
				}
			}
		};

		List.prototype.setStyle = function(key, value) {
			if (key && value) {
				this.style[key] = value;
			}
			this.$widget.css('width', this.style.width + 'px');
			this.$widget.css('height', this.style.height + 'px');

			this.height = this.style.height;
		};

		List.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.height
			};
		};
		
		List.prototype.resize = function() {
			this.width = this.window().width;
			this.style.width = this.width;
			this.setStyle();
		};

		return List;

	})();

	Widget.register('List', List);

	ListItem = (function() {

		function ListItem(data, parent) {
			this.data = data;
			this.parent = parent;
			this.page = parent.page;
			
			this.$li = $('<li></li>');
			this.$a = $('<a href="#" class="item-content item-link">').appendTo(this.$li);
			this.$itemInner = $('<div class="item-inner"></div>').appendTo(this.$a);
			this.$itemTitle = $('<div class="item-title"></div>').html(this.data.title).appendTo(this.$itemInner);
			this.$itemAfter = $('<div class="item-after"></div>').html(this.data.date).appendTo(this.$itemInner);
			
			var that = this;
			this.$li.click(function() {
				if (that.page) {
					FXUI.goPage(that.page);
				}
			});
			
			return this;
		}

		return ListItem;

	})();

	Widget.register('ListItem', ListItem);
	
	function ConverterExample(data) {
		this.data = data;
		this.convert = function() {
			return [ {
				title : 'Example',
				date : '2017-09-09'
			} ];
		};
		return this;
	}
	Converter['ConverterExample'] = ConverterExample;
	
	function Converter4GDGS(data) {
		this.data = data;
		this.convert = function() {
			var _data = [];
			for ( var i in this.data.items) {
				var item = this.data.items[i];
				_data.push({
					title : item.title,
					date : item.contentDate
				});
			}
			return _data;
		};
		return this;
	}
	Converter['Converter4GDGS'] = Converter4GDGS;

}).call(this);