(function() {
	'use strict';
	var $, Menu;

	$ = jQuery;

	Menu = (function() {

		Menu.description = {
			name : 'Menu',
			desc : '菜单'
		};

		function Menu(widget, parent) {
			Widget.apply(this, arguments);

			this.subWidgets = [];
			this.width = this.window().width;

			this.scroll = __bind(this.scroll, this);

			this.$widget = $('<div class="menu"></div>').attr('id', this.id);
			this.$nav = $('<div class="menu-nav row"></div>').appendTo(this.$widget);
			this.$header = $('#header');
			this.$content = $('.content', this.$panel);

			return this;
		}

		Menu.prototype.init = function() {
			this.parent.$widget.append(this.$widget);
			this.subWidgets.length > 0 && this.subWidgets[0].navItem.select();
			return this;
		};

		Menu.prototype.addSubWidget = function(widget) {
			this.subWidgets.push(widget);
		};

		Menu.prototype.exportWidget = function() {
			this.widget.widgets = [];
			for (var i = 0; i < this.subWidgets.length; i++) {
				this.subWidgets[i].exportWidget && this.widget.widgets.push(this.subWidgets[i].exportWidget());
			}
			return this.widget;
		};

		Menu.prototype.show = function() {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].show();
			}

			if (this.subWidgets.length <= 1) {
				this.$nav.remove();
				return;
			}

			this.subWidgets.length > 0 && this.subWidgets[0].navItem.selectItem();
			this.menuTop = this.getHeaderHeight() + this.$nav.height() + 10;
			this.$content.on('scroll', this.scroll);
		};

		Menu.prototype.hide = function() {
			this.hideNav();
			this.$content.scrollTop(0);
			this.$content.off('scroll', this.scroll);
		};

		Menu.prototype.layout = function() {
			this.parent.layout();
		};

		Menu.prototype.scroll = function() {
			var offsetTop = this.$widget.offset().top.toFixed(0) - this.getHeaderHeight();
			var offsetHeight = this.$widget[0].offsetHeight;
			if (offsetTop < 0 && -offsetTop < offsetHeight) {
				this.showNav();
			} else {
				this.hideNav();
			}
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				var top = this.subWidgets[i].$widget.offset().top;
				var height = this.subWidgets[i].$widget.height() - this.menuTop;
				if (top > -height && top <= this.menuTop) {
					if (this.$navCopy) {
						var $navCopies = $('.menu-nav-item', this.$navCopy);
						$navCopies.removeClass('active');
						$($navCopies[i]).addClass('active');
					}
					this.subWidgets[i].navItem.selectItem();
					break;
				}
			}
		};
		
		Menu.prototype.showNav = function() {
			if (!this.$navCopy) {
				var self = this;
				this.$navCopy = this.$nav.clone();
				this.$navCopy.addClass('topfixed');
				!this.$header.is(':visible') && this.$navCopy.css('top', 0);
				this.$content.before(this.$navCopy);
				var navItems = $('a', this.$navCopy);
				for (var i = 0, j = navItems.length; i < j; i++) {
					(function() {
						var index = i;
						$(navItems[index]).click(function() {
							$(this).siblings().removeClass('active');
							$(this).addClass('active');
							self.subWidgets[index].navItem.select();
						});
					})();
				}
			}
		};

		Menu.prototype.hideNav = function() {
			if (this.$navCopy) {
				this.$navCopy.remove();
				delete this.$navCopy;
			}
		};

		Menu.prototype.getHeaderHeight = function() {
			return this.$header.is(':visible') ? this.$header.height() : 0;
		};

		Menu.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.$widget.outerHeight(true)
			};
		};

		return Menu;

	})();

	Widget.register('Menu', Menu);

}).call(this);