(function() {
	'use strict';
	var $, MenuBlock, NavItem;

	$ = jQuery;

	MenuBlock = (function() {

		MenuBlock.example = {
			name : 'MenuBlock',
			title : '菜单块'
		};

		function MenuBlock(widget, parent) {
			Widget.apply(this, arguments);
			
			this.subWidgets = [];
			this.navItem = null;

			this.$widget = $('<div></div>').addClass('block row').attr('id', this.id);
			this.$header = $('<div></div>').addClass('block-header noselect').click(this.pack);
			this.$headerTitle = $('<h1></h1>').html(this.widget.title);
			this.$header.append(this.$headerTitle);
			this.$body = $('<div></div>').addClass('block-body');
			this.$widget.append(this.$header).append(this.$body);
			this.$content = $('.content', this.$panel);

			return this;
		}

		MenuBlock.prototype.init = function() {
			this.parent.$widget.append(this.$widget);
			this.navItem = new NavItem(this.parent, this);
			return this;
		};

		MenuBlock.prototype.addSubWidget = function(widget) {
			this.subWidgets.push(widget);
		};

		MenuBlock.prototype.exportWidget = function() {
			this.widget.widgets = [];
			for (var i = 0; i < this.subWidgets.length; i++) {
				this.subWidgets[i].exportWidget && this.widget.widgets.push(this.subWidgets[i].exportWidget());
			}
			return this.widget;
		};

		MenuBlock.prototype.show = function() {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].show();
			}
		};

		MenuBlock.prototype.layout = function() {
			this.parent.layout();
		};

		MenuBlock.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.$widget.outerHeight(true)
			};
		};

		return MenuBlock;

	})();

	NavItem = (function() {

		function NavItem(parent, block) {
			this.parent = parent;
			this.block = block;
			this.active = false;

			this.select = __bind(this.select, this);

			this.$widget = $('<a class="menu-nav-item"></a>').click(this.select);
			this.parent.$nav.append(this.$widget);
			this.index = $('a', this.parent.$nav).length - 1;

			this.setTitle(block.widget.title);
			
			return this;
		}

		NavItem.prototype.setTitle = function(title) {
			this.$widget.html(title);
		};

		NavItem.prototype.select = function() {
			this.scrollTo();
			this.selectItem();
		};

		NavItem.prototype.selectItem = function() {
			if (this.active) {
				return;
			}
			for (var i = 0, j = this.parent.subWidgets.length; i < j; i++) {
				this.parent.subWidgets[i].navItem.unselect();
			}
			this.active = true;
			this.$widget.addClass('active');
		};

		NavItem.prototype.unselect = function() {
			this.active = false;
			this.$widget.removeClass('active');
		};

		NavItem.prototype.scrollTo = function() {
			var positionTop = this.parent.$widget.position().top;
			var height = 0;
			for (var i = 0, j = this.index; i < j; i++) {
				var widget = this.parent.subWidgets[i];
				var marginTop = widget.$widget.css('margin-top');
				height += widget.$widget.height() + (marginTop ? parseInt(marginTop) : 0);
			}
			height += positionTop;
			this.block.$content.scrollTop(height);
		};

		return NavItem;

	})();

	Widget.register('MenuBlock', MenuBlock);

}).call(this);