(function() {
	'use strict';
	var $, Portal, storage = window.localStorage || {};

	$ = jQuery;

	Portal = (function() {

		function Portal() {
			this.contextPath = FXUI.getPlatformContextPath();
			this.appId = $.getUrlParam('FxAppId') || storage.FxAppId;
			this.platformType = FXUI.getPlatformType();
			this.pagekey = this.appId + '_' + this.platformType + '_page';
			this.rolekey = this.appId + '_role';
			this.page = $.getUrlParam('page') || storage[this.pagekey] || 'default';
			this.role = storage[this.rolekey];
			this.aikey = this.appId + '_appInfo';
			this.tplkey = this.appId + '_' + this.platformType + '_' + this.page + '_' + this.role;
			this.loadApp();
			// this.pushSession();
			return this;
		}
		
		Portal.prototype.loadApp = function() {
			if (storage[this.aikey] && storage[this.tplkey]) {
				this.runApp(JSON.parse(storage[this.aikey]), JSON.parse(storage[this.tplkey]), true);
				this.loadApp2(true);
			} else {
				this.loadApp2();
			}
		};

		Portal.prototype.loadApp2 = function(doInBackground) {
			var portal = this;
			var url = this.contextPath + '/index/loadApp.do?time=' + Date.now();
			var params = {
				appId : this.appId,
				platformType : this.platformType,
				page : this.page,
				role : $.getUrlParam('role'),
				FxAppId : this.appId
			};
			var callback = function(d) {
				if (d.flag == 'error') {
					Widget.run(blankTemplate);
					$.alert(d.errMsg, function() {
						FXUI.goPage(portal.contextPath + '/assets/user/user.html');
					});
					portal.loaded = true;
				} else {
					if (d.app != null && d.server != null && d.template != null) {
						var template = d.template;
						delete d.template;
						portal.runApp(d, JSON.parse(template.template), false);
						if (portal.isApp() && d.userDetails == null) {
							// portal.autoLogin();
						} else {
							portal.loaded = true;
						}
						if (d.verifyUrl) {
							storage.verifyUrl = d.verifyUrl;
						} else {
							storage.removeItem('verifyUrl');
						}
					} else {
						Widget.run(blankTemplate);
						portal.loaded = true;
					}
				}
			};
			FXUI.post(url, params, callback, doInBackground ? '' : '请稍候...');
		};
		
		Portal.prototype.runApp = function(appInfo, template, loadFromLocal) {
			this.userDetails = appInfo.userDetails;
			this.app = appInfo.app;
			this.server = appInfo.server;
			this.role = appInfo.role;
			this.tplkey = template.id;
			this.appId = this.app.appId;
			this.pagekey = this.appId + '_' + this.platformType + '_page';
			this.rolekey = this.appId + '_role';
			this.aikey = this.appId + '_appInfo';
			
			window.Portal = this;
			storage.FxAppId = appInfo.app.appId;
			storage[this.aikey] = JSON.stringify(appInfo);
			// storage[this.tplkey] = JSON.stringify(template);
			storage[this.pagekey] = this.page;
			storage[this.rolekey] = appInfo.role;

			if (loadFromLocal && template.widget.name === 'Link') {
				return;
			}
			this.template = template;
			this.page != 'default' ? $('title').html(template.name) : $('title').html(appInfo.app.name);
			Widget.run(template);
			template.notice && $.alert(template.notice, '\u7cfb\u7edf\u901a\u77e5');
		};

		Portal.prototype.isApp = function() {
			return this.platformType == '01' || this.platformType == '02';
		};
		
		Portal.prototype.goPage = function(page) {			
			if (page && page.blankTips) {
				$.toast(page.blankTips);
				return;
			}
			if (!page || !page.url) {
				return;
			}

			// this.setFunctionUsed(page.uuid);
			// 功能点击统计
			var params = {
				funMc : page.title,
				tplkey : this.tplkey,
				roleDm : this.role
			};
			$.post(this.contextPath + '/user/countFun.do', params);

			if (page.url.indexOf('miniProgram:') == 0) {
				var path = page.url.split(':')[1];
				FXAPP.navigateToMiniProgram(path);
				return;
			}

			if (page.url.indexOf('javascript:') == 0) {
				try {
					eval(page.url);
				} catch(e) {
					$.alert(e);
				}
				return;
			}

			storage['serverUrl'] = page.serverUrl;
			storage[this.pagekey] = page.page || this.page;
			var url = page.url;
			var prefix = '';
			if (url.indexOf('#') > 0 && url.indexOf('/#/') < 0) {
				prefix = url.substring(url.indexOf('#'), url.length);
				url = url.substring(0, url.indexOf('#'));
			}
			if (url.indexOf('?') > 0) {
				url += '&gdslxdm=' + page.serverType;
			} else {
				url += '?gdslxdm=' + page.serverType;
			}
			url += prefix;

			var self = this;
			var goLogin = function() {
				$.toast(page.redirectTips || '请先登录！');
				setTimeout(function() {
					FXUI.goPage('/app/assets/user/login.html?FxAppId=' + self.appId + '&page=' + self.page);
				}, 800);
			};

			var goMenu = function() {
				if (page.confirmText) {
					$.alert(page.confirmText, function() {
						FXUI.goPage(url);
					});
				} else {
					FXUI.goPage(url);
				}
			};

			switch (page.userType) {
			// 游客
			case '01':
				FXUI.goPage(url);
				break;
			// 登录
			case '02':
				if (this.userDetails != null) {
					goMenu(url);
				} else {
					goLogin();
				}
				break;
			// 企业
			case '03':
				if (this.userDetails != null) {
					if (this.userDetails.nsr != null) {
						goMenu(url);
					} else {
						var redirectUrl = '/app/assets/user/redirect.html?redirectUrl=' + encodeURIComponent(url)
							+ '&serverId=' + page.serverId + '&serverType=' + page.serverType;
						if (this.userDetails.userNsrs != null && this.userDetails.userNsrs.length > 0) {
							$.confirm('请先切换为企业纳税人身份！', function() {
								FXUI.goPage('/app/assets/user/role.html?redirectUrl=' + encodeURIComponent(redirectUrl));
							});
						} else {
							$.confirm('您尚未绑定纳税人，请先绑定纳税人！', function() {
								FXUI.goPage('/app/assets/user/bdnsr.html?redirectUrl=' + encodeURIComponent(redirectUrl));
							});
						}
					}
				} else {
					goLogin();
				}
				break;
			// 个人
			case '04':
				if (this.userDetails != null) {
					var redirectUrl = '/app/assets/user/redirect.html?redirectUrl='
							+ encodeURIComponent(url) + '&serverId=' + page.serverId + '&serverType=' + page.serverType;
					if (this.userDetails.zrr != null) {
						if (this.userDetails.role.sfDm == 'gr') {
							goMenu(url);
						} else {
							$.confirm('请先切换为个人身份！', function() {
								FXUI.goPage('/app/assets/user/role.html?redirectUrl=' + encodeURIComponent(redirectUrl));
							});
						}
					} else {
						$.confirm('您尚未实名认证，请先实名认证！', function() {
							FXUI.goPage('/app/assets/user/smzxx.html?redirectUrl=' + encodeURIComponent(redirectUrl));
						});
					}
				} else {
					goLogin();
				}
				break;
			default:
				FXUI.goPage(url);
				break;
			}
		};

		Portal.prototype.setFunctionUsed = function(id) {
			if (!id || this.userDetails == null || this.userDetails.user == null) {
				return;
			}
			var functionUsedKey = this.userDetails.user.yhid + '_function_used';
			var functionUsed = storage[functionUsedKey];
			functionUsed = functionUsed ? JSON.parse(functionUsed) : {};
			if (functionUsed[id]) {
				functionUsed[id] =  functionUsed[id] + 1;
			} else {
				functionUsed[id] = 1;
			}
			storage[functionUsedKey] = JSON.stringify(functionUsed);
		};

		Portal.prototype.logout = function() {
			var self = this;
			$.confirm('您确定退出登录吗？', function() {
				FXUI.post(self.contextPath + '/user/UserExit.do', {}, function(d) {
					if (d.flag != 'ok') {
						$.toast(d.errMsg);
					} else {
						storage.removeItem('currentAccount');
						storage.removeItem('mtax.nsr');
						FXUI.goPage('/app/assets/portal/index.html?FxAppId=' + self.app.appId);
					}
				}, '正在退出登录，请稍候...');
			});
		};

		Portal.prototype.loadCallback = function(callback) {
			if (this.loaded) {
				(typeof callback === 'function') && callback();
				return;
			}
			var self = this;
			setTimeout(function() {
				self.loadCallback(callback);
			}, 50);
		};

		Portal.prototype.pushSession = function() {
			storage.FxAppId = this.appId;
			FXUI.pushAllSessions(null, true);
		};
		
		return Portal;

	})();

	$(document).ready(function() {
		// alert(window.screen.width + '/' + window.screen.height + '/' + window.innerHeight + '/' + window.devicePixelRatio);
		window.Portal = new Portal();
		storage['appDebug'] == 'true' && (new VConsole());
	});
}).call(this);
