(function() {
	'use strict';
	var $, storage, searchWidgetMap;

	$ = jQuery;
	storage = window.localStorage;
	searchWidgetMap = {
		'GridItemV2' : true,
		'MenuItem' : true,
		'MenuListItem' : true
	};

	function Search() {
		this.searchHistoryKey = 'search_nav_search_history';
		this.$searchPanel = $('#search');
		this.$searchType = $('<div class="search-type-v"></div>');
		this.$searchTypeUl = $('<ul></ul>').appendTo(this.$searchType);
		this.$searchInput = $('.search-input', this.$searchPanel);
		this.$searchHistory = $('.hot-search-body>ul', this.$searchPanel);
		this.$searchWidget = $('.search-widget', this.$searchPanel);
		this.$overlay = $('<div class="panel-overlay"></div>');

		this.search = __bind(this.search, this);
		this.openSearchType = __bind(this.openSearchType, this);
		this.closeSearchType = __bind(this.closeSearchType, this);

		var self = this;
		this.$searchInput.on({
			'blur' : self.search,
			'keypress' : function() {
				window.event.keyCode == 13 && self.search();
			}
		});
		$('.search-type', this.$searchPanel).click(this.openSearchType);
		$('.icon2-down', this.$searchPanel).click(this.openSearchType);
	}

	Search.prototype.init = function() {
		this.showSearchHistory();
		this.initSearchType();
	};

	Search.prototype.initSearchType = function() {
		if (this.isInitSearchType) {
			return;
		}
		var self = this;
		var all = { title : '全部', uuid : 'all' };
		this.selectType = all;
		this.types = [];
		this.types.push(all);
		var resolve = function(widget) {
			if (widget.name === 'MenuBlock') {
				self.types.push(widget);
			}
			if (widget.widgets && widget.widgets.length > 0) {
				for (var i = 0, j = widget.widgets.length; i < j; i++) {
					resolve(widget.widgets[i]);
				}
			}
		};
		resolve(Portal.template.widget);
		for (var i = 0, j = this.types.length; i < j; i++) {
			(function() {
				var type = self.types[i];
				var $li = $('<li></li>').attr('uuid', type.uuid).html(type.title);
				$li.click(function() {
					$('li', self.$searchTypeUl).removeClass('selected');
					$(this).addClass('selected');
					self.doSelectType(type);
				});
				self.selectType.uuid == type.uuid && $li.addClass('selected');
				self.$searchTypeUl.append($li);
			})();
		}
		this.$searchType.css({
			height : (this.types.length * 2.2 + 0.4) + 'rem'
		});
		this.isInitSearchType = true;
	};

	Search.prototype.openSearchType = function() {
		var self = this;
		$('body').append(this.$overlay).append(this.$searchType);
		this.$searchType.slideToggle(300);
		this.$overlay.on('click', this.closeSearchType);
	};

	Search.prototype.closeSearchType = function() {
		var self = this;
		this.$searchType.slideToggle(300, function() {
			self.$searchType.detach();
			self.$overlay.off('click').detach();
		});
	};

	Search.prototype.doSelectType = function(type) {
		this.selectType = type;
		this.closeSearchType();
		$('.search-type', this.$searchPanel).html(this.selectType.title);
		this.search();
	};

	Search.prototype.search = function() {
		var word = this.$searchInput.val();
		$.showPreloader('搜索中...');
		this.searchWidgets = [];
		if (word) {
			if ((new RegExp("[`~!@#$^&*()\\-\\+=|{}':;',\\[\\].<>/?~！@#￥……&*（）—|{}【】‘；：”“'。，、？]")).test(word)) {
				this.$searchInput.val('');
				$.hidePreloader();
				$.toast('输入有误，请重新输入！');
				return;
			}
			this.saveSearchHistory(word);
			var widget = Widget.widget.widget;
			var targetWidget = this.selectType.uuid === 'all' ? widget : this.getMenuBlock(widget);
			targetWidget && this.searchWidget(word, targetWidget);
		} else {
			this.loadMenuBySelectType();
		}
		this.appendSearchWidget();
		$.hidePreloader();
	};

	Search.prototype.saveSearchHistory = function(word) {
		var searchHistory = storage[this.searchHistoryKey];
		searchHistory = searchHistory ? JSON.parse(searchHistory) : [];
		var index = -1;
		for (var i = 0, j = searchHistory.length; i < j; i++) {
			if (word === searchHistory[i]) {
				index = i;
				break;
			}
		}
		index > -1 && searchHistory.splice(index, 1);
		searchHistory.splice(0, 0, word);
		searchHistory.length > 5 && (searchHistory = searchHistory.slice(0, 5));
		storage[this.searchHistoryKey] = JSON.stringify(searchHistory);
		this.showSearchHistory();
	};

	Search.prototype.showSearchHistory = function() {
		var self = this;
		this.$searchHistory.empty();
		var searchHistory = storage[this.searchHistoryKey];
		searchHistory = searchHistory ? JSON.parse(searchHistory) : [];
		for (var i = 0, j = searchHistory.length; i < j; i++) {
			(function() {
				var sh = searchHistory[i];
				var $li = $('<li></li>').html(sh);
				$li.click(function() {
					self.$searchInput.val(sh);
					self.search();
				});
				self.$searchHistory.append($li);
			})();
		}
	};

	Search.prototype.getMenuBlock = function(widget) {
		if (widget.name === 'MenuBlock' && widget.uuid === this.selectType.uuid) {
			return widget;
		}
		if (widget.widgets && widget.widgets.length > 0) {
			for (var i = 0, j = widget.widgets.length; i < j; i++) {
				var ww = this.getMenuBlock(widget.widgets[i]);
				if (ww) {
					return ww;
				}
			}
		}
		return null;
	};

	Search.prototype.loadMenuBySelectType = function() {
		if (this.selectType.uuid === 'all') {
			return;
		}
		var self = this;
		var menuBlock = this.getMenuBlock(Widget.widget.widget);
		var resolve = function(widget) {
			if (widget.name === 'MenuItem') {
				self.searchWidgets.push(widget);
			}
			if (widget.widgets && widget.widgets.length > 0) {
				for (var i = 0, j = widget.widgets.length; i < j; i++) {
					resolve(widget.widgets[i], widget);
				}
			}
		};
		menuBlock && resolve(menuBlock);
	};

	Search.prototype.searchWidget = function(word, widget) {
		if (searchWidgetMap[widget.name] && widget.title && widget.title.indexOf(word) > -1) {
			this.searchWidgets.push(widget);
		}
		if (widget.widgets && widget.widgets.length > 0) {
			for (var i = 0, j = widget.widgets.length; i < j; i++) {
				this.searchWidget(word, widget.widgets[i]);
			}
		}
	};

	Search.prototype.appendSearchWidget = function() {
		this.$searchWidget.empty();
		for (var i = 0, j = this.searchWidgets.length; i < j; i++) {
			var searchRow = new SearchRow(this.searchWidgets[i]);
			this.$searchWidget.append(searchRow.$searchRow);
		}
	};
	
	function SearchRow(widget) {
		this.widget = widget;
		this.userType = widget.userType || '01';
		this.serverId = this.widget.serverId || Portal.app.serverId;
		this.serverType = '1';
		this.serverUrl = this.widget.serverUrl || Portal.server.gsServerUrl;

		this.open = __bind(this.open, this);

		this.$searchRow = $('<div class="block-row search-row"></div>').click(this.open);
		this.$searchIcon = $('<div class="search-icon"><img src="' + widget.icon + '"></div>');
		this.$searchText = $('<div class="search-text"><p>' + widget.title + '</p></div>');
		this.$searchRow.append(this.$searchIcon).append(this.$searchText);
		return this;
	}
	
	SearchRow.prototype.open = function() {
		Portal.goPage({
			uuid : this.widget.uuid,
			url : this.widget.url,
			serverId : this.serverId,
			serverUrl : this.serverUrl,
			serverType : this.serverType,
			userType : this.userType,
			redirectTips : this.widget.redirectTips,
            blankTips : this.widget.blankTips,
            title : this.widget.title
		});
	};

	window.Search = new Search();
}).call(this);