(function() {
	'use strict';
	var $, SearchNav, SearchRow, storage;

	$ = jQuery;
	storage = window.localStorage;

	SearchNav = (function() {

		function SearchNav(widget, parent) {
			Widget.apply(this, arguments);

			this.isWxConfig = false;

			this.scan = __bind(this.scan, this);
			this.openSearchPage = __bind(this.openSearchPage, this);
			this.search = __bind(this.search, this);

			this.$widget = $('<div class="searchnav"></div>');
			this.$header = $('#header');
			this.$scan = $('<span class="icon2-scan"></span>').click(this.scan);
			this.$search = $('<input class="search-input" type="text" placeholder="搜索">');
			this.$userHead = $('<div class="user-head"><img src="/static/images/common/head.png"/></div>');

			this.searchHistoryKey = 'search_nav_search_history';
			this.$searchPanel = $('#search');
			this.$searchInput = $('.search-input', this.$searchPanel);
			this.$searchHistory = $('.search-history>ul', this.$searchPanel);
			this.$searchWidget = $('.search-widget', this.$searchPanel);

			var that = this;
			this.$search.on({
				'click' : that.openSearchPage
			});
			this.$searchInput.on({
				'blur' : that.search,
				'keypress' : function() {
					window.event.keyCode == 13 && that.search();
				}
			});

			return this;
		}

		SearchNav.prototype.init = function() {
			this.$header.empty().append(this.$scan).append(this.$search).append(this.$userHead);
			this.loadUserHead();
			return this;
		};

		SearchNav.prototype.exportWidget = function() {
			this.widget.widgets = [];
			return this.widget;
		};

		SearchNav.prototype.scan = function() {
			var scanComplete = function(text) {
				if (text.indexOf('http') === 0 || text.indexOf('/') === 0) {
					window.location.href = text;
				} else {
					$.alert(text, '扫描结果');
				}
			};
			if (FXUI.isWx()) {
				var that = this;
				$.showIndicator();
				loadScript(window.location.protocol + '//res.wx.qq.com/open/js/jweixin-1.4.0.js', function() {
					var wxScan = function() {
						$.hideIndicator();
						wx.scanQRCode({
							needResult : 1,
							scanType : [ 'qrCode', 'barCode' ],
							success : function(res) {
								scanComplete(res.resultStr);
							}
						});
					};
					if (!that.isWxConfig) {
						FXUI.wxConfig(function() {
							that.isWxConfig = true;
							wxScan();
						});
					} else {
						wxScan();
					}
				});
			} else {
				FXAPP.scanQRCode({}, function(data) {
					scanComplete(data.text);
				});
			}
		};

		SearchNav.prototype.openSearchPage = function() {
			$.router.loadPage('#search');
			var that = this;
			setTimeout(function() {
				// that.$searchInput.focus();
				that.showSearchHistory();
			}, 200);
		};

		SearchNav.prototype.search = function() {
			var word = this.$searchInput.val();
			if (!word) {
				return;
			}
			$.showPreloader('搜索中...');
			this.saveSearchHistory(word);
			this.searchWidgets = [];
			var widget = Widget.widget.widget;
			this.searchWidget(word, widget);
			this.appendSearchWidget();
			$.hidePreloader();
		};

		SearchNav.prototype.saveSearchHistory = function(word) {
			var searchHistory = storage[this.searchHistoryKey];
			searchHistory = searchHistory ? JSON.parse(searchHistory) : [];
			var index = -1;
			for (var i = 0, j = searchHistory.length; i < j; i++) {
				if (word === searchHistory[i]) {
					index = i;
					break;
				}
			}
			index > -1 && searchHistory.splice(index, 1);
			searchHistory.splice(0, 0, word);
			searchHistory.length > 5 && (searchHistory = searchHistory.slice(0, 5));
			storage[this.searchHistoryKey] = JSON.stringify(searchHistory);
			this.showSearchHistory();
		};

		SearchNav.prototype.showSearchHistory = function() {
			var that = this;
			this.$searchHistory.empty();
			var searchHistory = storage[this.searchHistoryKey];
			searchHistory = searchHistory ? JSON.parse(searchHistory) : [];
			for (var i = 0, j = searchHistory.length; i < j; i++) {
				(function() {
					var sh = searchHistory[i];
					var $li = $('<li></li>').html(sh);
					$li.click(function() {
						that.$searchInput.val(sh);
						that.search();
					});
					that.$searchHistory.append($li);
				})();
			}
		};

		SearchNav.prototype.searchWidget = function(word, widget) {
			if (widget.name == 'GridItemV2' && widget.title && widget.title.indexOf(word) > -1) {
				this.searchWidgets.push(widget);
			}
			if (widget.widgets && widget.widgets.length > 0) {
				for (var i = 0, j = widget.widgets.length; i < j; i++) {
					this.searchWidget(word, widget.widgets[i]);
				}
			}
		};

		SearchNav.prototype.appendSearchWidget = function() {
			this.$searchWidget.empty();
			for (var i = 0, j = this.searchWidgets.length; i < j; i++) {
				var searchRow = new SearchRow(this.searchWidgets[i]);
				this.$searchWidget.append(searchRow.$searchRow);
			}
		};

		SearchNav.prototype.loadUserHead = function() {
			if (Portal.userDetails && Portal.userDetails.user && Portal.userDetails.user.yhtxdz) {
				var url = Portal.userDetails.user.yhtxdz;
				if (url.indexOf('http') < 0 && url.indexOf('/appres') < 0) {
					url = '/appres' + url;
				}
				this.$userHead.empty().append($('<img src="' + url + '"/>'));
			}
		};

		SearchNav.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.width,
				height : this.height
			};
		};

		return SearchNav;

	})();

	SearchRow = (function() {

		function SearchRow(widget) {
			this.widget = widget;
			this.userType = widget.userType || '01';
			this.serverId = this.widget.serverId || Portal.app.serverId;
			this.serverType = '1';
			this.serverUrl = this.widget.serverUrl || Portal.server.gsServerUrl;

			this.open = __bind(this.open, this);

			this.$searchRow = $('<div class="block-row search-row"></div>').click(this.open);
			this.$searchIcon = $('<div class="search-icon"><img src="' + widget.icon + '"></div>');
			this.$searchText = $('<div class="search-text"><p>' + widget.title + '</p></div>');
			this.$searchRow.append(this.$searchIcon).append(this.$searchText);
			return this;
		}
		
		SearchRow.prototype.open = function() {
			Portal.goPage({
				uuid : this.widget.uuid,
				url : this.widget.url,
				serverId : this.serverId,
				serverUrl : this.serverUrl,
				serverType : this.serverType,
				userType : this.userType,
                redirectTips : this.widget.redirectTips,
                title : this.widget.title
			});
		};

		return SearchRow;

	})();

	Widget.register('SearchNav', SearchNav);

}).call(this);