(function() {
	'use strict';
	var $, _document, TabBar, TabBarItem;

	$ = jQuery;
	_document = document;

	TabBar = (function() {

		function TabBar(widget, parent) {
			Widget.apply(this, arguments);

			this.subWidgets = [];
			this.title = widget.title;
			this.back = widget.back ? widget.back : false;
			this.userIcon = widget.userIcon ? widget.userIcon : false;
			this.switchTpl = widget.switchTpl ? widget.switchTpl : false;
			this.roleIcon = widget.roleIcon ? widget.roleIcon : false;
			this.separator = widget.separator ? widget.separator : false;
			this.taxType = widget.taxType;
			this.selectedTab = Widget.templateId + '_tab';
			this.draggingMoved = __bind(this.draggingMoved, this);
			this.draggingEnded = __bind(this.draggingEnded, this);
			this.draggingBegan = __bind(this.draggingBegan, this);
			this.position = __bind(this.position, this);
			this.layout = __bind(this.layout, this);
			this.switchTplChange = __bind(this.switchTplChange, this);

			this.$widget = $('<div></div>').addClass('bottom_bar');
			this.$header = $('#header');
			this.$footer = $('#footer');
			this.$back = $('<a class="button button-link button-nav pull-left"><span class="icon icon-left"></span></a>').click(function() {
				var tempurl = document.referrer;
				if (tempurl != null && tempurl.indexOf("/user/") > 0) {
					if (tempurl.indexOf("user.html") > 0) {
						$.router.back();
					} else {
						window.location.href = '/app/assets/portal/index.html?page=default';
					}
				} else {
					$.router.back();
				}
			});
			this.$userIcon = $('<a></a>').addClass('icon2-me').click(function() {
				FXUI.goPage(FXUI.getPlatformContextPath() + '/assets/user/user.html');
			});
			this.$switchTpl = $('<ul></ul>').addClass('switch');
			this.$switchTpll = $('<li>国</li>').attr('v', 'GS').appendTo(this.$switchTpl).click(this.switchTplChange);
			this.$switchTplr = $('<li>地</li>').attr('v', 'DS').appendTo(this.$switchTpl).click(this.switchTplChange);
			this.$roleIcon = $('<a></a>').addClass('icon2-role').click(function() {
				FXUI.goPage(FXUI.getPlatformContextPath() + '/assets/user/role.html');
			});
			this.$tabcontent = $('#tabcontent_container');
			this.$tabcontent.empty();
			this.fragment = _document.createDocumentFragment();

			this.style = widget.style || {};
			this.style.background = this.style.background || '#22289e';
			this.style.selBackground = this.style.selBackground || '#21279c';
			this.style.fontColor = this.style.fontColor || '#efefef';
			this.style.selFontColor = this.style.selFontColor || '#56abe3';
			this.style.separatorColor = this.style.separatorColor || '#b2b2b2';
			this.style.headerBackground = this.style.headerBackground || '';
			this.style.headerBackgroundImg = this.style.headerBackgroundImg || '';
			this.style.headerFontColor = this.style.headerFontColor || '#ffffff';
			this.style.bodyBackground = this.style.bodyBackground || '';
			this.style.bodyBackgroundImg = this.style.bodyBackgroundImg || '';
			this.style.backColor = this.style.backColor || '#ffffff';
			this.style.userIconColor = this.style.userIconColor || '#ffffff';
			this.style.switchTplColor = this.style.switchTplColor || '#e0e0e0';
			this.style.switchTplColorOn = this.style.switchTplColorOn || '#0092ce';
			this.style.roleIconColor = this.style.roleIconColor || '#ffffff';

			return this;
		}

		TabBar.prototype.init = function() {
			this.append();
			this.setStyle();
			this.initDraggable();
			var tabIndex = Widget.storage[this.selectedTab] || 0;
			tabIndex = tabIndex >= this.subWidgets.length ? 0 : tabIndex;
			var aTabIndex = $.getUrlParam('tabIndex');
			if (aTabIndex && !isNaN(parseInt(aTabIndex))) {
				tabIndex = parseInt(aTabIndex);
			}
			if (this.subWidgets.length > 0) {
				var a = this;
				setTimeout(function() {
					a.subWidgets[tabIndex].performClick();
				}, 100);
			}
			return this;
		};

		TabBar.prototype.addSubWidget = function(widget) {
			this.subWidgets.push(widget);
		};

		TabBar.prototype.exportWidget = function() {
			this.sortSubWidgets();
			this.widget.widgets = [];
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				if (typeof this.subWidgets[i].exportWidget === 'function') {
					this.widget.widgets.push(this.subWidgets[i].exportWidget());
				}
			}
			this.widget.style = this.style;
			return this.widget;
		};

		TabBar.prototype.elements = function() {
			return this.$widget.children();
		};

		TabBar.prototype.initDraggable = function(method) {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				if (this.subWidgets[i].name == 'TabContent') {
					break;
				}
				new Draggable(this.subWidgets[i], {
					began : this.draggingBegan,
					ended : this.draggingEnded,
					moved : this.draggingMoved
				});
			}
			this.layout();
		};

		TabBar.prototype.appendDropContainer = function() {
			this.$dropContainer = $('<div></div>').addClass('drop_container');
			this.$dropContainer.css({
				width : 0,
				height : 0
			});
			this.$widget.append(this.$dropContainer);
		};

		TabBar.prototype.removeDropContainer = function() {
			this.$dropContainer.remove();
		};

		TabBar.prototype.draggingBegan = function(event, $target) {
			this.appendDropContainer();
			this.sort(this.elements());
		};

		TabBar.prototype.draggingEnded = function(event, $target) {
			if (this.$dragging) {
				this.$dragging.data('position', this.$dropContainer.data('position'));
			}
			this.$dragging = undefined;
			this.removeDropContainer();
			setTimeout(this.layout, 0);
		};

		TabBar.prototype.draggingMoved = function(event, $target) {
			var $dragging = $target;
			this.$dropContainer.css({
				width : $dragging.width() + 2,
				height : $dragging.height()
			}).addClass('active');
			var distance = parseInt(($dragging.position().left + 20) / $dragging.width());

			var $sortedEles = this.sort(this.elements());
			distance = distance < 0 ? 0 : distance;
			distance = distance >= $sortedEles.length ? ($sortedEles.length - 1) : distance;
			this.$dropContainer.data('position', distance);
			var $newEles = [];
			for (var i = 0, j = $sortedEles.length; i < j; i++) {
				var $element = $($sortedEles[i]);
				if (!$element.is('.drop_container')) {
					$newEles.push($element);
				}
			}
			$newEles.splice(distance, 0, this.$dropContainer);
			for (var i = 0, j = $newEles.length; i < j; i++) {
				var $element = $($newEles[i]);
				$element.data('position', i);
			}
			this.$dragging = $dragging;
			this.layout();
		};

		TabBar.prototype.position = function($elements) {
			var eleWidth = this.window().width / $elements.length;
			var positions = [];
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				positions.push({
					x : eleWidth * i,
					y : 1,
					w : eleWidth,
					h : $element.height(),
					$element : $element
				});
			}
			return positions;
		};

		TabBar.prototype.layout = function() {
			var $elements = this.sort(this.elements());
			var positions = this.position($elements);
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				var position = positions[i];
				if ($element.is('.dragging')) {
					continue;
				}
				$element.css({
					position : 'absolute',
					left : position.x,
					top : position.y,
					width : position.w
				});
			}
		};

		TabBar.prototype.findTabBarItem = function(position) {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				if (this.subWidgets[i].data4position() == position) {
					return this.subWidgets[i];
				}
			}
		};

		TabBar.prototype.switchTplChange = function(event) {
			if (this.$switchTpll.is('.active')) {
				this.$switchTpll.removeClass('active').css('background', 'none');
				this.$switchTplr.addClass('active');
				this.$switchTplr.css('background', this.style.switchTplColorOn);
				this.taxType = this.$switchTplr.attr('v');
			} else if (this.$switchTplr.is('.active')) {
				this.$switchTplr.removeClass('active').css('background', 'none');
				this.$switchTpll.addClass('active');
				this.$switchTpll.css('background', this.style.switchTplColorOn);
				this.taxType = this.$switchTpll.attr('v');
			}
			this.widget.taxType = this.taxType;
			Portal.loadTaxTypeTemplate(this.taxType);
		};

		TabBar.prototype.openTab = function(index) {
			this.subWidgets[index].performClick();
		};

		TabBar.prototype.setStyle = function(key, value) {
			if (key && value) {
				this.style[key] = value;
			}
			this.$widget.css('background-color', this.style.background);
			if (this.style.headerBackground) {
				this.$header.css('background', this.style.headerBackground);
			}
			if (this.style.headerBackgroundImg) {
				var background = 'url(' + this.style.headerBackgroundImg + ') no-repeat center';
				this.$header.css('background', background);
				this.$header.css('background-size', 'cover');
			}
			$('h1', this.$header).css('color', this.style.headerFontColor);
			if (this.style.bodyBackground) {
				$('#content').css('background', this.style.bodyBackground);
			}
			if (this.style.bodyBackgroundImg) {
				var background = 'url(' + this.style.bodyBackgroundImg + ') no-repeat center';
				$('#content').css('background', background);
				$('#content').css('background-size', 'cover');
			}
			$('span', this.$back).css('color', this.style.backColor);
			this.$userIcon.css('color', this.style.userIconColor);
			if (this.switchTpl && this.taxType) {
				this.$switchTpl.css('background', this.style.switchTplColor);
				$('li.active', this.$switchTpl).css('background', this.style.switchTplColorOn);
			}
			this.$roleIcon.css('color', this.style.roleIconColor);
			this.$widget.css('border-top', this.separator ? ('1px solid ' + this.style.separatorColor) : 'none');
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].setStyle(key, value);
			}
		};

		TabBar.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.$widget.width(),
				height : this.$widget.height()
			};
		};

		TabBar.prototype.resize = function() {
			this.layout();
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].resize();
			}
		};

		TabBar.prototype.append = function() {
			$('#header>h1').empty();
			if (this.back) {
				this.$back.prependTo($('#header'));
			}
			$('.icon2-me', $('#header')).remove();
			if (this.userIcon) {
				this.$userIcon.prependTo($('#header'));
			}
			$('.switch', $('#header')).remove();
			if (this.switchTpl && this.taxType) {
				this.$switchTpl.prependTo($('#header'));
				$('li[v="' + this.taxType + '"]', this.$switchTpl).addClass('active');
			}
			if (this.roleIcon) {
				this.$roleIcon.prependTo($('#header'));
			}
			this.$widget.append(this.fragment);
			this.$footer.empty().append(this.$widget);
		};

		return TabBar;

	})();

	Widget.register('TabBar', TabBar);

	TabBarItem = (function() {

		function TabBarItem(widget, parent) {
			Widget.apply(this, arguments);

			this.selected = false;
			this.login = widget.login == 'Y';
			this.headerHidden = widget.headerHidden || false;
			this.$title = $('<p></p>').html(widget.title);
			this.$badge = $('<span>12</span>').addClass('badge');
			this.$icon = $('<img />').attr('src', widget.icon);
			this.$iconSelected = $('<img />').attr('src', widget.iconSelected);
			this.$iconWrapper = $('<div></div>').addClass('bar_item_icon').append(this.$icon);
			this.$widget = $('<div></div>').addClass('bar_item').append(this.$iconWrapper).append(this.$title);
			this.$tabcontent = parent.$tabcontent;
			this.fragment = _document.createDocumentFragment();

			this.style = parent.style || {};

			return this;
		}

		TabBarItem.prototype.init = function() {
			var a = this;
			// this.parent.$widget.append(this.$widget);
			this.parent.fragment.appendChild(this.$widget[0]);
			this.$tabcontent.append(this.fragment);
			this.$widget.bind('click', function(event) {
				event.preventDefault();
				event.stopPropagation();
				if (a.selected) {
					a.subWidget.show();
					return;
				}
				if (a.login && Portal.userDetails == null) {
					FXUI.checkIsLogin(true);
				} else {
					a.select();
				}
			});
			this.setStyle();
			this.showBadge();
			return this;
		};

		TabBarItem.prototype.addSubWidget = function(widget) {
			this.subWidget = widget;
		};

		TabBarItem.prototype.exportWidget = function() {
			this.widget.widgets = [];
			if (typeof this.subWidget.exportWidget === 'function') {
				this.widget.widgets.push(this.subWidget.exportWidget());
			}
			this.widget.style = this.style;
			return this.widget;
		};

		TabBarItem.prototype.performClick = function() {
			this.$widget.click();
		};

		TabBarItem.prototype.select = function() {
			if (this.widget.url) {
				FXUI.goPage(this.widget.url);
				return;
			}
			for (var i = 0, j = this.parent.subWidgets.length; i < j; i++) {
				var tbItem = this.parent.subWidgets[i];
				tbItem.selected = false;
				tbItem.$widget.removeClass('bar_item_on');
				tbItem.$iconSelected.detach();
				tbItem.$iconWrapper.append(tbItem.$icon);
				tbItem.setStyle();
				tbItem.subWidget.hide();
			}
			this.selected = true;
			$('#header>h1').html(this.widget.title);
			this.$icon.detach();
			this.$iconWrapper.append(this.$iconSelected);
			this.$widget.addClass('bar_item_on');
			this.setHeader();
			this.setStyle();
			this.subWidget.show();
			Widget.storage[this.parent.selectedTab] = this.data4position();
		};
		
		TabBarItem.prototype.setHeader = function() {
			if (this.headerHidden) {
				this.parent.$header.hide();
				this.parent.$tabcontent.closest('.content').css('top', 0);
			} else {
				this.parent.$header.show();
				this.parent.$tabcontent.closest('.content').css('top', '2.2rem');
			}
		};

		TabBarItem.prototype.data4position = function() {
			return parseInt(this.$widget.data('position'));
		};

		TabBarItem.prototype.setStyle = function(key, value) {
			if (key != undefined && value != undefined) {
				this.style[key] = value;
			}
			this.$widget.css('background-color', this.style.background);
			this.$title.css('color', this.style.fontColor);
			if (this.selected) {
				this.$widget.css('background-color', this.style.selBackground);
				this.$title.css('color', this.style.selFontColor);
			}
		};

		TabBarItem.prototype.showBadge = function() {
			var a = this;
			var url = this.widget.badgeUrl;
			if (!url) {
				return;
			}
			var callback = function(d) {
				if (d.number && d.number > 0) {
					a.$badge.html(d.number > 99 ? 99 : d.number);
					a.$iconWrapper.append(a.$badge);
				}
			};
			FXUI.post(url, {}, callback);
		};

		TabBarItem.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.$widget.width(),
				height : this.$widget.height()
			};
		};

		TabBarItem.prototype.resize = function() {
			this.subWidget.resize();
		};

		return TabBarItem;

	})();

	Widget.register('TabBarItem', TabBarItem);

}).call(this);