(function() {
	'use strict';
	var $, TabContent;

	$ = jQuery;

	TabContent = (function() {

		function TabContent(widget, parent) {
			Widget.apply(this, arguments);

			this.subWidgets = [];
			this.subWidgetsMap = {};
			this.draggingMoved = __bind(this.draggingMoved, this);
			this.draggingEnded = __bind(this.draggingEnded, this);
			this.draggingBegan = __bind(this.draggingBegan, this);
			this.position = __bind(this.position, this);
			this.layout = __bind(this.layout, this);

			this.$widget = $('<div></div>').attr('id', this.id).addClass('tab_content');
			this.$header = $('#header');
			this.$footer = $('#footer');
			this.$tabcontent = this.parent.$tabcontent;

			return this;
		}

		TabContent.prototype.init = function() {
			if (this.parent.name === 'TabBar') {
				this.$tabcontent.append(this.$widget);
			} else if (this.parent.name === 'TabBarItem') {
				this.parent.fragment.appendChild(this.$widget[0]);
			}
			this.setFooter();
			this.initDraggable();
			return this;
		};

		TabContent.prototype.addSubWidget = function(widget) {
			this.subWidgets.push(widget);
			this.subWidgetsMap[widget.id] = widget;
		};

		TabContent.prototype.exportWidget = function() {
			this.sortSubWidgets();
			this.widget.widgets = [];
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				if (typeof this.subWidgets[i].exportWidget === 'function') {
					this.widget.widgets.push(this.subWidgets[i].exportWidget());
				}
			}
			return this.widget;
		};

		TabContent.prototype.setFooter = function() {
			if (this.parent.name == 'TabBarItem') {
				this.$footer.show();
			} else {
				this.$footer.hide();
			}
		};

		TabContent.prototype.performClick = function() {
			if (this.parent.name == 'TabBar' && this.parent.title) {
				$('#header>h1').html(this.parent.title);
			}
			this.show();
		};

		TabContent.prototype.show = function() {
			this.$widget.show();
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].show && this.subWidgets[i].show();
			}
			this.resize();
		};

		TabContent.prototype.hide = function() {
			this.$widget.hide();
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				this.subWidgets[i].hide && this.subWidgets[i].hide();
			}
		};

		TabContent.prototype.elements = function() {
			return this.$widget.children();
		};

		TabContent.prototype.initDraggable = function(method) {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				new Draggable(this.subWidgets[i], {
					began : this.draggingBegan,
					ended : this.draggingEnded,
					moved : this.draggingMoved
				});
			}
			this.layout();
		};

		TabContent.prototype.draggingBegan = function(event, $target, widget) {
			this.appendDropContainer();
			widget.draggingBegan && widget.draggingBegan();
			this.sort(this.elements());
		};

		TabContent.prototype.draggingEnded = function(event, $target, widget) {
			if (this.$dragging) {
				this.$dragging.data('position', this.$dropContainer.data('position'));
			}
			this.$dragging = undefined;
			this.removeDropContainer();
			widget.draggingEnded && widget.draggingEnded();
			setTimeout(this.layout, 0);
		};

		TabContent.prototype.draggingMoved = function(event, $target, widget) {
			var $dragging = $target;
			this.$dropContainer.css({
				width : $dragging.width(),
				height : $dragging.height()
			}).addClass('active');
			var $sortedEles = this.sort(this.elements());
			var distance = parseInt($dragging.data('position'));
			for (var i = 0, j = $sortedEles.length; i < j; i++) {
				var $element = $($sortedEles[i]);
				if ($dragging.position().top > $element.position().top) {
					var dh = $element.height() - $dragging.height();
					var dd = $dragging.position().top - $element.position().top;
					if (dh > 0 && dd < dh && distance <= i) {
						continue;
					}
					if (distance > i && dd > 30) {
						continue;
					}
					distance = i;
				}
			}

			distance = distance < 0 ? 0 : distance;
			distance = distance >= $sortedEles.length ? ($sortedEles.length - 1)
					: distance;
			$dragging.data('position', distance);
			this.$dropContainer.data('position', distance);
			var $newEles = [];
			for (var i = 0, j = $sortedEles.length; i < j; i++) {
				var $element = $($sortedEles[i]);
				if (!$element.is('.drop_container')) {
					$newEles.push($element);
				}
			}
			$newEles.splice(distance, 0, this.$dropContainer);
			for (var i = 0, j = $newEles.length; i < j; i++) {
				var $element = $($newEles[i]);
				$element.data('position', i);
			}
			this.$dragging = $dragging;
			widget.draggingMoved && widget.draggingMoved();
			this.layout();
		};

		TabContent.prototype.position = function($elements) {
			var positions = [];
			var y = 0;
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				var aWidget = this.subWidgetsMap[$element.attr('id')];
				var frame = (aWidget && aWidget.frame) ? aWidget.frame() : undefined;
				positions.push({
					x : 0,
					y : y,
					w : frame ? frame.width : $element.width(),
					h : frame ? frame.height : $element.height(),
					$element : $element
				});
				y += frame ? frame.height : $element.height();
			}
			return positions;
		};

		TabContent.prototype.layout = function() {
			var height = 0;
			var $elements = this.sort(this.elements());
			var positions = this.position($elements);
			for (var i = 0, j = $elements.length; i < j; i++) {
				var $element = $($elements[i]);
				var position = positions[i];
				if ($element.is('.dragging')) {
					continue;
				}
				$element.css({
					position : 'absolute',
					left : position.x,
					top : position.y
				});
				height += $element.height();
			}
			this.$widget.height(height);
		};

		TabContent.prototype.setStyle = function(key, value) {

		};

		TabContent.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.$widget.width(),
				height : this.$widget.height()
			};
		};

		TabContent.prototype.resize = function() {
			for (var i = 0, j = this.subWidgets.length; i < j; i++) {
				if (typeof this.subWidgets[i].resize === 'function') {
					this.subWidgets[i].resize();
				}
			}
			this.layout();
		};

		TabContent.prototype.appendDropContainer = function() {
			this.$dropContainer = $('<div></div>').addClass('drop_container');
			this.$dropContainer.css({
				width : 0,
				height : 0
			});
			this.$widget.append(this.$dropContainer);
		};

		TabContent.prototype.removeDropContainer = function() {
			this.$dropContainer.remove();
		};

		return TabContent;

	})();

	Widget.register('TabContent', TabContent);

}).call(this);