(function() {
	'use strict';
	var $, UsedAlot, storage = window.localStorage;

	$ = jQuery;

	UsedAlot = (function() {

		function UsedAlot(widget, parent) {
			GridItemV2.apply(this, arguments);

			this.parent.inited = false;

			return this;
		}

		UsedAlot.prototype.init = function() {
			if (!Portal.loaded) {
				var self = this;
				setTimeout(function() {
					self.init();
				}, 50);
				self.parent.inited = false;
				return;
			}
			if (Portal.userDetails == null) {
				this.parent.inited = true;
				return;
			}
			var a = this;
			if (a.parent.row && a.parent.elements().length >= a.parent.row * a.parent.column) {
				this.parent.inited = true;
				return;
			}
			a.parent.$container.append(a.$widget);
			a.$widget.bind('click', function(event) {
				event.preventDefault();
				event.stopPropagation();
				a.open();
			});
			this.initDragSwith();
			this.parent.inited = true;
			return this;
		};

		UsedAlot.prototype.reInit = function(widget) {
			this.widget = widget;
			this.blank = widget.title == '' && widget.icon == '';
			this.$title.html(widget.title);
			if (!this.blank && widget.icon) {
				this.$icon.append($('<img />').attr('src', widget.icon));
			} else {
				this.$icon.empty();
				this.$widget.unbind('click');
			}
			this.isChanged = true;
			this.$widget.data('blank', this.blank);
		};

		UsedAlot.prototype.exportWidget = function() {
			return this.widget;
		};

		UsedAlot.prototype.initDragSwith = function() {
			this.offEvents();
			var self = this;
			var c;
			var reset = function() {
				c && clearTimeout(c);
				c = 0;
			};
			this.$widget.on('mousedown touchstart', function() {
				c = setTimeout(function() {
					reset();
					self.parent.turnOnDrag();
				}, 2000);
			});
			this.$widget.on('mousemove touchmove', function() {
				reset();
			});
			this.$widget.on('mouseup touchend touchcancel', function() {
				reset();
			});
		};

		UsedAlot.prototype.offEvents = function() {
			this.$widget.off('mousedown touchstart mousemove touchmove mouseup touchend touchcancel');
		};

		UsedAlot.prototype.shake = function() {
			if (!this.widget.draggable) {
				return;
			}
			var self = this;
			var $img = $('img', this.$icon);
			var r1 = function() {
				$img.removeClass('r-right').addClass('r-left');
				self._shake = setTimeout(function() {
					r2();
				}, 100);
			};
			var r2 = function() {
				$img.removeClass('r-left').addClass('r-right');
				self._shake = setTimeout(function() {
					r1();
				}, 100);
			};
			r1();
		};

		UsedAlot.prototype.unshake = function() {
			$('img', this.$icon).removeClass('r-left').removeClass('r-right');
			this._shake && clearTimeout(this._shake);
		};

		UsedAlot.prototype.show = function() {
			if (!this.blank && this.widget.icon && !this.imgLoaded) {
				this.$icon.append($('<img />').attr('src', this.widget.icon));
				this.imgLoaded = true;
			}
		};
		
		UsedAlot.prototype.open = function() {
			if (!this.parent.isDragOn) {
				$.showIndicator();
				loadScript('/static/js/assets/portal/hme.min.js', function() {
					$.hideIndicator();
					$.router.loadPage('#menuedit');
					setTimeout(function() {
						HomeMenuEdit.init();
					}, 300);
				});
			}
		};

		UsedAlot.prototype.performClick = function() {
			this.open();
		};

		UsedAlot.prototype.setStyle = function(key, value) {
			if (key != undefined && value != undefined) {
				this.style[key] = value;
			}
			this.$title.css('color', this.style.fontColor);
			if (this.border) {
				var borderStyle = '1px solid ' + this.style.borderColor;
				var position = this.frame().$widget.data('position');
				var col = this.parent.column;
				var x = position % col;
				var y = parseInt(position / col);
				this.frame().$widget.addClass('bg');
				this.frame().$widget.css('border-top', y == 0 ? borderStyle : 'none');
				this.frame().$widget.css('border-right', x == col - 1 ? 'none' : borderStyle);
				this.frame().$widget.css('border-bottom', borderStyle);
				if (this.frame().$widget.is('.dragging')) {
					this.frame().$widget.css('border', 'none');
				}
			}
		};

		UsedAlot.prototype.frame = function() {
			return {
				$widget : this.$widget,
				top : this.$widget.position().top,
				left : this.$widget.position().left,
				width : this.$widget.width(),
				height : this.$widget.height()
			};
		};

		UsedAlot.prototype.resize = function() {
			this.$icon.css({
				width : this.parent.colWidth * 0.32,
				height : this.parent.colWidth * 0.32,
				'margin-top' : this.parent.colWidth * 0.254
			});
			this.$title.css({
				width : this.parent.colWidth * 0.75
			});
		};

		return UsedAlot;

	})();

	Widget.register('UsedAlot', UsedAlot);

}).call(this);