(function() {
	'use strict';
	var $, index, Widget, Draggable, Draggable2, __bind = function(fn, me) {
		return function() {
			return fn.apply(me, arguments);
		};
	};

	$ = jQuery;

	index = 1000;

	window.__bind = __bind;

	$.getUrlParam = function(name) {
		var regExp = new RegExp("(^|&)" + name + "=([^&]*)(&|$)");
		var r = window.location.search.substr(1).match(regExp);
		if (r != null) {
			return unescape(r[2]);
		}
		return null;
	}

	Widget = (function() {

		Widget.version, Widget.widget, Widget.widgets = {},
				Widget.servers = {}, Widget.sessions = {},
				Widget.storage = window.localStorage || {};

		Widget.register = function(name, widget) {
			Widget.widgets[name] = widget;
			window[name] = widget;
		};

		Widget.run = function(template) {
			var lastVersion = Widget.storage[template.id + '_version'];
			var lastTemplateId = Widget.storage['templateId'];
			if (lastVersion && lastVersion === template.version && lastTemplateId == template.id && Widget.ran) {
				return;
			}
			var widget = template.widget;
			if (Widget.storage[template.id]) {
				var tpl = JSON.parse(Widget.storage[template.id]);
				if (template.version === tpl.version) {
					widget = tpl.widget;
				}
			}
			Widget.templateId = template.id;
			Widget.version = template.version;
			Widget.widget = Widget.resolve(widget);
			Widget.ran = true;
			Widget.storage[template.id] = JSON.stringify(template);
			Widget.storage[template.id + '_version'] = Widget.version;
			Widget.storage['templateId'] = Widget.templateId;
		};

		Widget.resolve = function(widget, parent) {
			if (Widget.widgets[widget.name] == undefined) {
				console.log(widget.name + ' is not registered.');
				return null;
			}
			var w = new Widget.widgets[widget.name](widget, parent);
			if (widget.widgets && widget.widgets.length > 0) {
				for (var i = 0, j = widget.widgets.length; i < j; i++) {
					var ww = Widget.resolve(widget.widgets[i], w);
					(ww && w.addSubWidget) && w.addSubWidget(ww);
				}
			}
			w.init && w.init();
			return w;
		};

		Widget.save = function() {
			var widget = Widget.widget.exportWidget();
			var tpl = {
				id : Widget.templateId,
				version : Widget.version,
				widget : widget
			};
			Widget.storage[Widget.templateId] = JSON.stringify(tpl);
		};

		function Widget() {
			this.widget = arguments[0];
			this.parent = arguments[1];
			this.$panel = $('#home');
			this.id = this.widget.name + index;
			this.name = this.widget.name;
			this.index = index++;
			this.draggable = this.widget.draggable;

			this.window = function() {
				return {
					width : $(window).width(),
					height : $(window).height()
				};
			};

			this.sort = function($elements) {
				for (var i = 0, j = $elements.length; i < j; i++) {
					var $element = $($elements[i]);
					var position = $element.data('position');
					if (position == null) {
						$element.data('position', i);
					}
				}
				var $sortedElements = $elements.not('.dragging');
				for (var i = 0, l = $sortedElements.length; i < l; i++) {
					var $e1 = $($sortedElements[i]);
					for (var j = i + 1, ll = $sortedElements.length; j < ll; j++) {
						var $e2 = $($sortedElements[j]);
						var e1p = $e1.data('position');
						var e2p = $e2.data('position');
						if (e1p != null && e2p != null && parseInt(e1p) > parseInt(e2p)) {
							$sortedElements[i] = $e2;
							$sortedElements[j] = $e1;
							$e1 = $e2;
						}
					}
				}
				return $sortedElements;
			};

			this.sortSubWidgets = function() {
				for (var i = 0, l = this.subWidgets.length; i < l; i++) {
					var sw1 = this.subWidgets[i];
					for (var j = i + 1, ll = this.subWidgets.length; j < ll; j++) {
						var sw2 = this.subWidgets[j];
						var sw1p = sw1.frame().$widget.data('position');
						var sw2p = sw2.frame().$widget.data('position');
						if (sw1p != null && sw2p != null && parseInt(sw1p) > parseInt(sw2p)) {
							this.subWidgets[i] = sw2;
							this.subWidgets[j] = sw1;
							sw1 = sw2;
						}
					}
				}
			};

		}

		return Widget;

	})();

	window.Widget = Widget;

	Draggable = (function() {

		function Draggable(widget, callbacks) {
			if (typeof widget.draggable === 'boolean' && !widget.draggable) {
				return;
			}
			this.click = __bind(this.click, this);
			this.moved = __bind(this.moved, this);
			this.ended = __bind(this.ended, this);
			this.began = __bind(this.began, this);
			this.coordinate = __bind(this.coordinate, this);
			this.off = __bind(this.off, this);
			this.on = __bind(this.on, this);
			this.toggle = __bind(this.toggle, this);
			this.bind = __bind(this.bind, this);
			this.widget = widget;
			this.$container = widget.frame().$widget;
			this.callbacks = callbacks;
			this.toggle();
		}

		Draggable.prototype.bind = function(method) {
			if (method == null) {
				method = 'on';
			}
			$(document)[method]('mousemove touchmove', this.moved);
			return $(document)[method]('mouseup touchend touchcancel',
					this.ended);
		};

		Draggable.prototype.toggle = function(method) {
			if (method == null) {
				method = 'on';
			}
			this.$container[method]('mousedown touchstart', this.began);
		};

		Draggable.prototype.on = function() {
			return this.toggle('on');
		};

		Draggable.prototype.off = function() {
			return this.toggle('off');
		};

		Draggable.prototype.coordinate = function(event) {
			switch (event.type) {
			case 'touchstart':
			case 'touchmove':
			case 'touchend':
			case 'touchcancel':
				if (event instanceof TouchEvent) {
					return event.touches[0];
				}
				return event.originalEvent.touches[0];
			default:
				return event;
			}
		};

		Draggable.prototype.began = function(event) {
			var _ref;
			if (this.$target) {
				return;
			}
			if (typeof this.widget.draggable === 'boolean'
					&& !this.widget.draggable) {
				return;
			}
			event.preventDefault();
			event.stopPropagation();
			this.isMove = false;
			this.bind('on');
			this.wPosition = this.$container.data('position');
			this.$target = this.$container;
			this.origin = {
				x : this.coordinate(event).pageX - this.$target.position().left,
				y : this.coordinate(event).pageY - this.$target.position().top
			};
			this.touchPisition = {
				x : this.coordinate(event).pageX,
				y : this.coordinate(event).pageY
			};
			return (_ref = this.callbacks) != null ? typeof _ref.began === 'function' ? _ref
					.began(event, this.$target, this.widget)
					: void 0
					: void 0;
		};

		Draggable.prototype.ended = function(event) {
			if (!this.isMove) {
				this.widget.performClick && this.widget.performClick();
			}
			var _ref;
			if (this.$target == null) {
				return;
			}
			event.preventDefault();
			event.stopPropagation();
			this.bind('off');
			this.$target.removeClass('dragging');
			delete this.$target;
			delete this.origin;
			delete this.touchPisition;
			var r = (_ref = this.callbacks) != null ? typeof _ref.ended === 'function' ? _ref
					.ended(event, this.$target, this.widget)
					: void 0
					: void 0;
			if (this.wPosition != this.$container.data('position')
					|| this.widget.isChanged) {
				Widget.save();
			}
			return r;
		};

		Draggable.prototype.moved = function(event) {
			var _ref;
			if (this.$target == null) {
				return;
			}
			event.preventDefault();
			event.stopPropagation();
			this.isMove = Math.abs(this.coordinate(event).pageX - this.touchPisition.x) > 2 || Math.abs(this.coordinate(event).pageY - this.touchPisition.y) > 2;
			if (!this.isMove) {
				return;
			}
			this.$target.addClass('dragging');
			this.$target.css({
				position : 'absolute',
				left : this.coordinate(event).pageX - this.origin.x,
				top : this.coordinate(event).pageY - this.origin.y
			});
			this.dragged = this.$target;
			return (_ref = this.callbacks) != null ? typeof _ref.moved === 'function' ? _ref
					.moved(event, this.$target, this.widget)
					: void 0
					: void 0;
		};

		return Draggable;

	})();

	window.Draggable = Draggable;

	Draggable2 = (function() {

		function Draggable2($target, callbacks) {
			this.moved = __bind(this.moved, this);
			this.ended = __bind(this.ended, this);
			this.began = __bind(this.began, this);
			this.coordinate = __bind(this.coordinate, this);
			this.toggle = __bind(this.toggle, this);
			this.bind = __bind(this.bind, this);
			this.$target = $target;
			this.callbacks = callbacks;
			this.toggle();
		}

		Draggable2.prototype.bind = function(method) {
			if (method == null) {
				method = 'on';
			}
			$(document)[method]('mousemove touchmove', this.moved);
			return $(document)[method]('mouseup touchend touchcancel', this.ended);
		};

		Draggable2.prototype.toggle = function(method) {
			if (method == null) {
				method = 'on';
			}
			this.$target[method]('mousedown touchstart', this.began);
		};

		Draggable2.prototype.coordinate = function(event) {
			switch (event.type) {
			case 'touchstart':
			case 'touchmove':
			case 'touchend':
			case 'touchcancel':
				if (event instanceof TouchEvent) {
					return event.touches[0];
				}
				return event.originalEvent.touches[0];
			default:
				return event;
			}
		};

		Draggable2.prototype.began = function(event) {
			event.preventDefault();
			event.stopPropagation();
			this.isMove = false;
			this.bind('on');
			this.origin = {
				x : this.coordinate(event).pageX - this.$target.offset().left,
				y : this.coordinate(event).pageY - this.$target.offset().top
			};
			this.touchPisition = {
				x : this.coordinate(event).pageX,
				y : this.coordinate(event).pageY
			};
			var _ref;
			return (_ref = this.callbacks) != null ? typeof _ref.began === 'function' ? _ref
					.began(event, this.$target)
					: void 0
					: void 0;
		};

		Draggable2.prototype.ended = function(event) {
			event.preventDefault();
			event.stopPropagation();
			this.isMove = false;
			this.bind('off');
			delete this.origin;
			delete this.touchPisition;
			var self = this;
			this.$dragging && this.$dragging.toggle(400, function() {
				self.$dragging.remove();
				delete self.$dragging;
			});
			var _ref;
			return (_ref = this.callbacks) != null ? typeof _ref.ended === 'function' ? _ref
					.ended(event, this.$target)
					: void 0
					: void 0;
		};

		Draggable2.prototype.moved = function(event) {
			event.preventDefault();
			event.stopPropagation();
			this.isMove = Math.abs(this.coordinate(event).pageX - this.touchPisition.x) > 6 || Math.abs(this.coordinate(event).pageY - this.touchPisition.y) > 6;
			if (this.isMove && this.$dragging == null) {
				this.$dragging = $('<div></div>').append(this.$target.clone());
				this.$dragging.css({
					position : 'absolute',
					left : this.$target.offset().left,
					top : this.$target.offset().top
				});
				$('body').append(this.$dragging);
			}
			if (this.$dragging == null) {
				return;
			}
			this.$dragging.addClass('x-dragging active');
			this.$dragging.css({
				position : 'absolute',
				left : this.coordinate(event).pageX - this.origin.x,
				top : this.coordinate(event).pageY - this.origin.y
			});
			var _ref;
			return (_ref = this.callbacks) != null ? typeof _ref.moved === 'function' ? _ref
					.moved(event, this.$target, this.$dragging)
					: void 0
					: void 0;
		};

		return Draggable2;

	})();

	window.Draggable2 = Draggable2;

	$(window).resize(function() {
		Widget.widget && Widget.widget.resize && Widget.widget.resize();
	});

}).call(this);