(function() {
	'use strict';
	var provinces = {
		"11" : {
			code : "110000",
			text : "北京市"
		},
		"12" : {
			code : "120000",
			text : "天津市"
		},
		"13" : {
			code : "130000",
			text : "河北省"
		},
		"14" : {
			code : "140000",
			text : "山西省"
		},
		"15" : {
			code : "150000",
			text : "内蒙古自治区"
		},
		"21" : {
			code : "210000",
			text : "辽宁省"
		},
		"22" : {
			code : "220000",
			text : "吉林省"
		},
		"23" : {
			code : "230000",
			text : "黑龙江省"
		},
		"31" : {
			code : "310000",
			text : "上海市"
		},
		"32" : {
			code : "320000",
			text : "江苏省"
		},
		"33" : {
			code : "330000",
			text : "浙江省"
		},
		"34" : {
			code : "340000",
			text : "安徽省"
		},
		"35" : {
			code : "350000",
			text : "福建省"
		},
		"36" : {
			code : "360000",
			text : "江西省"
		},
		"37" : {
			code : "370000",
			text : "山东省"
		},
		"41" : {
			code : "410000",
			text : "河南省"
		},
		"42" : {
			code : "420000",
			text : "湖北省"
		},
		"43" : {
			code : "430000",
			text : "湖南省"
		},
		"44" : {
			code : "440000",
			text : "广东省"
		},
		"45" : {
			code : "450000",
			text : "广西壮族自治区"
		},
		"46" : {
			code : "460000",
			text : "海南省"
		},
		"50" : {
			code : "500000",
			text : "重庆市"
		},
		"51" : {
			code : "510000",
			text : "四川省"
		},
		"52" : {
			code : "520000",
			text : "贵州省"
		},
		"53" : {
			code : "530000",
			text : "云南省"
		},
		"54" : {
			code : "540000",
			text : "西藏自治区"
		},
		"61" : {
			code : "610000",
			text : "陕西省"
		},
		"62" : {
			code : "620000",
			text : "甘肃省"
		},
		"63" : {
			code : "630000",
			text : "青海省"
		},
		"64" : {
			code : "640000",
			text : "宁夏回族自治区"
		},
		"65" : {
			code : "650000",
			text : "新疆维吾尔自治区"
		},
		"71" : {
			code : "710000",
			text : "台湾省"
		},
		"81" : {
			code : "810000",
			text : "香港特别行政区"
		},
		"82" : {
			code : "820000",
			text : "澳门特别行政区"
		}
	};
	var idcardLen = 18;
	var ratioValue = 11;
	var ratio = [ 7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2 ]; // 身份证号码前17位数分别乘以不同的系数
	var matchMap = {
		0 : '1',
		1 : '0',
		2 : 'X',
		3 : '9',
		4 : '8',
		5 : '7',
		6 : '6',
		7 : '5',
		8 : '4',
		9 : '3',
		10 : '2'
	};
	var regex = {
		phone : /^((086-)?0[0-9]{2,3}-[0-9]{7,8}\;)*((086-)?0[0-9]{2,3}-[0-9]{7,8})$/,
		email : /^[a-z0-9_]+(\.?[a-z0-9-_])*?@([a-zA-Z0-9]([a-zA-Z0-9\-]*?[a-zA-Z0-9])?\.)+[a-zA-Z]{2,20}$/i,
		mobile : /(^(13\d|14[57]|15\d|16\d|17\d|18\d|19\d)\d{8})$/,
		domain : /^[a-z0-9\u4e00-\u9fa5]+([\-]+[a-z0-9\u4e00-\u9fa5]+)*((\.[0-9a-z]{1,10}){1,2})$/i,
		ip : /^([0-9]|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.([0-9]|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.([0-9]|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.([0-9]|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])$/,
		password : /^(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])(?=.*[^A-Za-z0-9]).{8,}$/,
		uppercase : /[A-Z]/,
		lowercase : /[a-z]/,
		number : /[0-9]/,
		integer : /^\d+$/,
		character : /[^A-Za-z0-9]/,
		companyLicenseNumber : /^\d{15}$/,
		organizationNumber : /^[A-Za-z0-9]{8}-[A-Za-z0-9]{1}$/,
		socialCode : /^[A-Za-z0-9]{18}$/,
		url : /^(https?:\/\/)?((([a-z\d]([a-z\d-]*[a-z\d])*)\.)+[a-z]{2,}|((\d{1,3}\.){3}\d{1,3}))|(localhost)(\:\d+)?(\/[-a-z\d%_.~+]*)*(\?[;&a-z\d%_.~+=-]*)?(\#[-a-z\d_]*)?$/i,
		bankAccount : /^\d{8,35}$/,
		isCardNo : /(^\d{15}$)|(^\d{18}$)|(^\d{17}(\d|X)$)/
	};

	function isType(obj, type) {
		return Object.prototype.toString.call(obj) === '[object ' + type + ']';
	}

	function isObject(obj) {
		return isType(obj, 'Object');
	}

	function isString(str) {
		return isType(str, 'String');
	}

	function isFunction(fn) {
		return isType(fn, 'Function');
	}

	function isNumber(num) {
		return isType(num, 'Number');
	}

	function leapYear(year) {
		if (year % 400 === 0 || (year % 4 === 0 && year % 100 !== 0)) {
			return true;
		} else {
			return false;
		}
	}

	function inRange(val, min, max) {
		if (val >= min && val <= max) {
			return true;
		}
		return false;
	}

	function verifyBirthday(birthday) {
		// 验证生日合法性
		if (!isString(birthday) || birthday.length !== 8) {
			return false;
		}
		var year = birthday.slice(0, 4), month = birthday.slice(4, 6), date = birthday
				.slice(6, 8);
		var flag = false;
		if (isNaN(Number(year)) || isNaN(Number(month)) || isNaN(Number(date))) {
			return false;
		}
		year = Number(year), month = Number(month), date = Number(date);
		if (year > new Date().getFullYear()) {
			return false;
		}
		var FebDay = 28;
		if (leapYear(year)) {
			FebDay = 29;
		}
		switch (month) {
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			flag = inRange(date, 1, 31);
			break;
		case 2:
			flag = inRange(date, 1, FebDay);
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			flag = inRange(date, 1, 30);
			break;
		}
		return flag;
	}

	function hit(idcard) {
		return hitArea(idcard) ? true : false;
	}

	function hitArea(idcard) {
		if (!isString(idcard) || idcard.length !== 18) {
			return false;
		}
		var areaCode = idcard.slice(0, 2);
		if (provinces[areaCode]) {
			return provinces[areaCode]
		}
		return false;
	}

	function genLastChar(pre17Char) {
		var pre17CharArr = pre17Char.split('');
		var lastChar = 0;
		for (var i = 0; i < pre17CharArr.length; i++) {
			lastChar += pre17CharArr[i] * ratio[i];
		}
		lastChar = matchMap[lastChar % 11];
		return lastChar;
	}

	function upgrade15To18(card) {
		var eighteenCard;
		card = card.toString();
		if (card.length === 15) {
			eighteenCard = card.slice(0, 6) + '19' + card.slice(6);
			eighteenCard += genLastChar(eighteenCard);
			return {
				code : 0,
				msg : '身份证15位升级到18位成功',
				card : eighteenCard
			};
		} else {
			return {
				code : -1,
				msg : '不是合法的15位身份证',
				card : card
			};
		}
	}

	function isIdcard(idcard) {
		if (!isString(idcard)) {
			return false;
		}
		if (idcard.length !== idcardLen) {
			return false;
		}
		// 验证生日合法性
		if (!verifyBirthday(idcard.slice(6, 14))) {
			return false;
		}
		var lastChar = idcard.slice(idcardLen - 1);
		var pre17Char = idcard.slice(0, idcardLen - 1);
		var charList = pre17Char.split('');
		var totleCount = 0;
		if (!/^(\d|X)$/.test(lastChar)) {
			return false;
		}
		return hit(idcard);
		//210202199709090366校验不通过
		// 将这17位数字和系数相乘的结果相加,对11进行求余，得出身份证最后一个字符
		/*for (var i = 0, len = charList.length; i < len; i++) {
			if (isNaN(Number(charList[i]))) {
				return false;
			}
			totleCount += Number(charList[i]) * ratio[i];
		}
		if (matchMap[totleCount % ratioValue] === lastChar) {
			return hit(idcard);
		} else {
			return false;
		}*/
	}
	
	function isCardNo(card) {  
        // 身份证号码为15位或者18位，15位时全为数字，18位前17位为数字，最后一位是校验位，可能为数字或字符X  
        var reg = regex.isCardNo;  
        if(!reg.test(card)) {
            //alert("身份证输入不合法");  
            return  false;  
        }
        return true;
    }  

	function isPhone(phone) {
		return regex.phone.test(phone);
	}

	function isEmail(email) {
		return regex.email.test(email);
	}

	function isMobile(mobile) {
		return regex.mobile.test(mobile);
	}

	function isDomain(domain) {
		return regex.domain.test(domain);
	}

	function isIp(ip) {
		return regex.ip.test(ip);
	}
    window.isCardNo = isCardNo;
	window.isIdcard = isIdcard;
	window.isPhone = isPhone;
	window.isEmail = isEmail;
	window.isMobile = isMobile;
	window.isDomain = isDomain;
	window.isIp = isIp;
}.call(this));